@@ -7,6 +7,9 @@ from _ctypes.basics import sizeof, byref, as_ffi_pointer
 from _ctypes.array import Array, array_get_slice_params, array_slice_getitem,\
      array_slice_setitem
 
+try: from __pypy__ import builtinify
+except ImportError: builtinify = lambda f: f
+
 # This cache maps types to pointers to them.
 _pointer_type_cache = {}
 
@@ -78,7 +81,9 @@ class _Pointer(_CData):
         addr = self._buffer[0]
         if addr == 0:
             raise ValueError("NULL pointer access")
-        return self._type_.from_address(addr)
+        instance = self._type_.from_address(addr)
+        instance.__dict__['_base'] = self
+        return instance
 
     def setcontents(self, value):
         if not isinstance(value, self._type_):
@@ -154,6 +159,7 @@ def _cast_addr(obj, _, tp):
 
     return result
 
+@builtinify
 def POINTER(cls):
     try:
         return _pointer_type_cache[cls]
@@ -173,6 +179,7 @@ def POINTER(cls):
         _pointer_type_cache[cls] = klass
     return klass
 
+@builtinify
 def pointer(inst):
     return POINTER(type(inst))(inst)
 
