@@ -16,6 +16,7 @@
 ######################################################################
 
 import numpypy
+import _numpypy
 
 # Module containing non-deprecated functions borrowed from Numeric.
 __docformat__ = "restructuredtext en"
@@ -274,7 +275,7 @@ def choose(a, choices, out=None, mode='raise'):
             [-1, -2, -3, -4, -5]]])
 
     """
-    raise NotImplementedError('Waiting on interp level method')
+    return _numpypy.choose(a, choices, out, mode)
 
 
 def repeat(a, repeats, axis=None):
@@ -316,7 +317,7 @@ def repeat(a, repeats, axis=None):
            [3, 4]])
 
     """
-    raise NotImplementedError('Waiting on interp level method')
+    return _numpypy.repeat(a, repeats, axis)
 
 
 def put(a, ind, v, mode='raise'):
@@ -1290,7 +1291,9 @@ def clip(a, a_min, a_max, out=None):
     array([3, 4, 2, 3, 4, 5, 6, 7, 8, 8])
 
     """
-    raise NotImplementedError('Waiting on interp level method')
+    if not hasattr(a, 'clip'):
+        a = numpypy.array(a)
+    return a.clip(a_min, a_max, out=out)
 
 
 def sum(a, axis=None, dtype=None, out=None):
@@ -1360,10 +1363,9 @@ def sum(a, axis=None, dtype=None, out=None):
 
     """
     assert dtype is None
-    assert out is None
     if not hasattr(a, "sum"):
         a = numpypy.array(a)
-    return a.sum(axis=axis)
+    return a.sum(axis=axis, out=out)
 
 
 def product (a, axis=None, dtype=None, out=None):
@@ -1720,11 +1722,11 @@ def amax(a, axis=None, out=None):
     4.0
 
     """
-    assert axis is None
-    assert out is None
     if not hasattr(a, "max"):
         a = numpypy.array(a)
-    return a.max()
+    if a.size < 1:
+        return numpypy.array([])
+    return a.max(axis=axis, out=out)
 
 
 def amin(a, axis=None, out=None):
@@ -1782,12 +1784,11 @@ def amin(a, axis=None, out=None):
     0.0
 
     """
-    # amin() is equivalent to min()
-    assert axis is None
-    assert out is None
     if not hasattr(a, 'min'):
         a = numpypy.array(a)
-    return a.min()
+    if a.size < 1:
+        return numpypy.array([])
+    return a.min(axis=axis, out=out)
 
 def alen(a):
     """
