@@ -778,6 +778,40 @@ class TestCompiler:
             raise AssertionError("attribute not removed")"""
         yield self.st, test, "X.__name__", "X"
 
+    def test_lots_of_loops(self):
+        source = "for x in y: pass\n" * 1000
+        compile_with_astcompiler(source, 'exec', self.space)
+
+    def test_assign_to_empty_list_1(self):
+        source = """if 1:
+        for i in range(5):
+            del []
+            [] = ()
+            [] = []
+            [] = [] = []
+        ok = 1
+        """
+        self.simple_test(source, 'ok', 1)
+
+    def test_assign_to_empty_list_2(self):
+        source = """if 1:
+        for i in range(5):
+            try: [] = 1, 2, 3
+            except ValueError: pass
+            else: raise AssertionError
+            try: [] = a = 1
+            except TypeError: pass
+            else: raise AssertionError
+            try: [] = _ = iter(['foo'])
+            except ValueError: pass
+            else: raise AssertionError
+            try: [], _ = iter(['foo']), 1
+            except ValueError: pass
+            else: raise AssertionError
+        ok = 1
+        """
+        self.simple_test(source, 'ok', 1)
+
 
 class AppTestCompiler:
 
@@ -805,6 +839,13 @@ class AppTestCompiler:
          assert s.getvalue() == "hi lovely!"
          """ in {}
 
+    def test_assert_with_tuple_arg(self):
+        try:
+            assert False, (3,)
+        except AssertionError, e:
+            assert str(e) == "(3,)"
+        
+
 class TestOptimizations:
     def count_instructions(self, source):
         code, blocks = generate_function_code(source, self.space)
