@@ -1,8 +1,7 @@
 import sys
 from pypy.interpreter.error import OperationError
-from pypy.rlib.rarithmetic import LONG_BIT
-from pypy.rlib.unroll import unrolling_iterable
-from pypy.rlib import jit
+from rpython.rlib.unroll import unrolling_iterable
+from rpython.rlib import jit
 
 TICK_COUNTER_STEP = 100
 
@@ -40,6 +39,7 @@ class ExecutionContext(object):
     def gettopframe(self):
         return self.topframeref()
 
+    @jit.unroll_safe
     def gettopframe_nohidden(self):
         frame = self.topframeref()
         while frame and frame.hide():
@@ -144,7 +144,10 @@ class ExecutionContext(object):
         actionflag = self.space.actionflag
         if actionflag.get_ticker() < 0:
             actionflag.action_dispatcher(self, frame)     # slow path
-    bytecode_trace_after_exception._always_inline_ = True
+    bytecode_trace_after_exception._always_inline_ = 'try'
+    # NB. this function is not inlined right now.  backendopt.inline would
+    # need some improvements to handle this case, but it's not really an
+    # issue
 
     def exception_trace(self, frame, operationerr):
         "Trace function called upon OperationError."
@@ -153,17 +156,13 @@ class ExecutionContext(object):
             self._trace(frame, 'exception', None, operationerr)
         #operationerr.print_detailed_traceback(self.space)
 
-    def _convert_exc(self, operr):
-        # Only for the flow object space
-        return operr
-
     def sys_exc_info(self): # attn: the result is not the wrapped sys.exc_info() !!!
         """Implements sys.exc_info().
         Return an OperationError instance or None."""
         frame = self.gettopframe_nohidden()
         while frame:
             if frame.last_exception is not None:
-                return self._convert_exc(frame.last_exception)
+                return frame.last_exception
             frame = self.getnextframe_nohidden(frame)
         return None
 
@@ -344,9 +343,13 @@ class AbstractActionFlag(object):
         signal, the tick counter is set to -1 by C code in signals.h.
         """
         assert isinstance(action, PeriodicAsyncAction)
-        self._periodic_actions.append(action)
+        # hack to put the release-the-GIL one at the end of the list,
+        # and the report-the-signals one at the start of the list.
         if use_bytecode_counter:
+            self._periodic_actions.append(action)
             self.has_bytecode_counter = True
+        else:
+            self._periodic_actions.insert(0, action)
         self._rebuild_action_dispatcher()
 
     def getcheckinterval(self):
@@ -420,15 +423,6 @@ class AsyncAction(object):
         The action must have been registered at space initalization time."""
         self.space.actionflag.fire(self)
 
-    def fire_after_thread_switch(self):
-        """Bit of a hack: fire() the action but only the next time the GIL
-        is released and re-acquired (i.e. after a potential thread switch).
-        Don't call this if threads are not enabled.  Currently limited to
-        one action (i.e. reserved for CheckSignalAction from module/signal).
-        """
-        from pypy.module.thread.gil import spacestate
-        spacestate.action_after_thread_switch = self
-
     def perform(self, executioncontext, frame):
         """To be overridden."""
 
