@@ -5,12 +5,7 @@ from pypy.interpreter.error import OperationError
 from pypy.interpreter.baseobjspace import W_Root
 import os, sys
 
-import inspect
-
 class MixedModule(Module):
-
-    NOT_RPYTHON_ATTRIBUTES = ['loaders']
-
     applevel_name = None
     expose__file__attribute = True
 
@@ -94,13 +89,13 @@ class MixedModule(Module):
             return None
         else:
             w_value = loader(space)
-            func = space.interpclass_w(w_value)
             # the idea of the following code is that all functions that are
             # directly in a mixed-module are "builtin", e.g. they get a
             # special type without a __get__
             # note that this is not just all functions that contain a
             # builtin code object, as e.g. methods of builtin types have to
             # be normal Functions to get the correct binding behaviour
+            func = w_value
             if (isinstance(func, Function) and
                 type(func) is not BuiltinFunction):
                 try:
@@ -124,14 +119,11 @@ class MixedModule(Module):
             self.w_initialdict = space.call_method(self.w_dict, 'items')
         return self.w_dict
 
-    def _freeze_(self):
+    def _cleanup_(self):
         self.getdict(self.space)
         self.w_initialdict = None
         self.startup_called = False
         self._frozen = True
-        # hint for the annotator: Modules can hold state, so they are
-        # not constant
-        return False
 
     def buildloaders(cls):
         """ NOT_RPYTHON """
