@@ -1,25 +1,24 @@
 """ PyFrame class implementation with the interpreter main loop.
 """
 
-from pypy.tool.pairtype import extendabletype
-from pypy.interpreter import eval, baseobjspace, pycode
+from rpython.tool.pairtype import extendabletype
+from pypy.interpreter import eval, pycode
 from pypy.interpreter.argument import Arguments
 from pypy.interpreter.error import OperationError, operationerrfmt
 from pypy.interpreter.executioncontext import ExecutionContext
 from pypy.interpreter import pytraceback
-from pypy.rlib.objectmodel import we_are_translated, instantiate
-from pypy.rlib.jit import hint
-from pypy.rlib.debug import make_sure_not_resized, check_nonneg
-from pypy.rlib.rarithmetic import intmask, r_uint
-from pypy.rlib import jit
+from rpython.rlib.objectmodel import we_are_translated, instantiate
+from rpython.rlib.jit import hint
+from rpython.rlib.debug import make_sure_not_resized, check_nonneg
+from rpython.rlib.rarithmetic import intmask, r_uint
+from rpython.rlib import jit
 from pypy.tool import stdlib_opcode
-from pypy.tool.stdlib_opcode import host_bytecode_spec
+from rpython.tool.stdlib_opcode import host_bytecode_spec
 
 # Define some opcodes used
-g = globals()
 for op in '''DUP_TOP POP_TOP SETUP_LOOP SETUP_EXCEPT SETUP_FINALLY
 POP_BLOCK END_FINALLY'''.split():
-    g[op] = stdlib_opcode.opmap[op]
+    globals()[op] = stdlib_opcode.opmap[op]
 HAVE_ARGUMENT = stdlib_opcode.HAVE_ARGUMENT
 
 class PyFrame(eval.Frame):
@@ -304,11 +303,17 @@ class PyFrame(eval.Frame):
     @jit.dont_look_inside
     def descr__reduce__(self, space):
         from pypy.interpreter.mixedmodule import MixedModule
-        from pypy.module._pickle_support import maker # helper fns
         w_mod    = space.getbuiltinmodule('_pickle_support')
         mod      = space.interp_w(MixedModule, w_mod)
         new_inst = mod.get('frame_new')
-        w        = space.wrap
+        w_tup_state = self._reduce_state(space)
+        nt = space.newtuple
+        return nt([new_inst, nt([]), w_tup_state])
+
+    @jit.dont_look_inside
+    def _reduce_state(self, space):
+        from pypy.module._pickle_support import maker # helper fns
+        w = space.wrap
         nt = space.newtuple
 
         cells = self._getcells()
@@ -359,8 +364,7 @@ class PyFrame(eval.Frame):
             w(self.instr_prev_plus_one),
             w_cells,
             ]
-
-        return nt([new_inst, nt([]), nt(tup_state)])
+        return nt(tup_state)
 
     @jit.dont_look_inside
     def descr__setstate__(self, space, w_args):
