@@ -1,6 +1,6 @@
 from pypy.interpreter.error import OperationError
 from pypy.interpreter import unicodehelper
-from pypy.rlib.rstring import StringBuilder
+from rpython.rlib.rstring import StringBuilder
 
 def parsestr(space, encoding, s, unicode_literal=False):
     """Parses a string or unicode literal, and return a wrapped value.
@@ -91,19 +91,20 @@ def parsestr(space, encoding, s, unicode_literal=False):
         assert 0 <= bufp <= bufq
         substr = buf[bufp:bufq]
         if rawmode:
-            v = unicodehelper.PyUnicode_DecodeRawUnicodeEscape(space, substr)
+            v = unicodehelper.decode_raw_unicode_escape(space, substr)
         else:
-            v = unicodehelper.PyUnicode_DecodeUnicodeEscape(space, substr)
+            v = unicodehelper.decode_unicode_escape(space, substr)
         return space.wrap(v)
 
     need_encoding = (encoding is not None and
-                     encoding != "utf-8" and encoding != "iso-8859-1")
+                     encoding != "utf-8" and encoding != "utf8" and
+                     encoding != "iso-8859-1")
     assert 0 <= ps <= q
     substr = s[ps : q]
     if rawmode or '\\' not in s[ps:]:
         if need_encoding:
-            w_u = space.wrap(unicodehelper.PyUnicode_DecodeUTF8(space, substr))
-            w_v = unicodehelper.PyUnicode_AsEncodedString(space, w_u, space.wrap(encoding))
+            w_u = space.wrap(unicodehelper.decode_utf8(space, substr))
+            w_v = unicodehelper.encode(space, w_u, encoding)
             return w_v
         else:
             return space.wrap(substr)
@@ -129,19 +130,18 @@ def PyString_DecodeEscape(space, s, recode_encoding):
     builder = StringBuilder(len(s))
     ps = 0
     end = len(s)
-    while 1:
-        ps2 = ps
-        while ps < end and s[ps] != '\\':
+    while ps < end:
+        if s[ps] != '\\':
+            # note that the C code has a label here.
+            # the logic is the same.
             if recode_encoding and ord(s[ps]) & 0x80:
                 w, ps = decode_utf8(space, s, ps, end, recode_encoding)
+                # Append bytes to output buffer.
                 builder.append(w)
-                ps2 = ps
             else:
+                builder.append(s[ps])
                 ps += 1
-        if ps > ps2:
-            builder.append_slice(s, ps2, ps)
-        if ps == end:
-            break
+            continue
 
         ps += 1
         if ps == end:
@@ -219,8 +219,8 @@ def decode_utf8(space, s, ps, end, encoding):
     # while (s < end && *s != '\\') s++; */ /* inefficient for u".."
     while ps < end and ord(s[ps]) & 0x80:
         ps += 1
-    w_u = space.wrap(unicodehelper.PyUnicode_DecodeUTF8(space, s[pt:ps]))
-    w_v = unicodehelper.PyUnicode_AsEncodedString(space, w_u, space.wrap(encoding))
+    w_u = space.wrap(unicodehelper.decode_utf8(space, s[pt:ps]))
+    w_v = unicodehelper.encode(space, w_u, encoding)
     v = space.str_w(w_v)
     return v, ps
 
