@@ -1,19 +1,19 @@
-from pypy.conftest import gettestobjspace
-from pypy.rpython.lltypesystem import rffi
-from pypy.rlib.clibffi import get_libc_name
-from pypy.rlib.libffi import types
-from pypy.rlib.libffi import CDLL
-from pypy.rlib.test.test_clibffi import get_libm_name
+from rpython.rtyper.lltypesystem import rffi
+from rpython.rlib.clibffi import get_libc_name
+from rpython.rlib.libffi import types
+from rpython.rlib.libffi import CDLL
+from rpython.rlib.test.test_clibffi import get_libm_name
 
 import sys, py
 
 class BaseAppTestFFI(object):
+    spaceconfig = dict(usemodules=('_ffi', '_rawffi'))
 
     @classmethod
     def prepare_c_example(cls):
-        from pypy.tool.udir import udir
-        from pypy.translator.tool.cbuild import ExternalCompilationInfo
-        from pypy.translator.platform import platform
+        from rpython.tool.udir import udir
+        from rpython.translator.tool.cbuild import ExternalCompilationInfo
+        from rpython.translator.platform import platform
 
         c_file = udir.ensure("test__ffi", dir=1).join("foolib.c")
         # automatically collect the C source from the docstrings of the tests
@@ -37,8 +37,7 @@ class BaseAppTestFFI(object):
         return str(platform.compile([c_file], eci, 'x', standalone=False))
 
     def setup_class(cls):
-        space = gettestobjspace(usemodules=('_ffi', '_rawffi'))
-        cls.space = space
+        space = cls.space
         cls.w_iswin32 = space.wrap(sys.platform == 'win32')
         cls.w_libfoo_name = space.wrap(cls.prepare_c_example())
         cls.w_libc_name = space.wrap(get_libc_name())
@@ -97,7 +96,7 @@ class AppTestFFI(BaseAppTestFFI):
         libm = CDLL(self.libm_name)
         pow_addr = libm.getaddressindll('pow')
         fff = sys.maxint*2-1
-        if sys.platform == 'win32':
+        if sys.platform == 'win32' or sys.platform == 'darwin':
             fff = sys.maxint*2+1
         assert pow_addr == self.pow_addr & fff
 
@@ -627,4 +626,17 @@ class AppTestFFI(BaseAppTestFFI):
                             types.void, FUNCFLAG_STDCALL)
         sleep(10)
 
- 
+    def test_by_ordinal(self):
+        """
+            int DLLEXPORT AAA_first_ordinal_function()
+            {
+                return 42;
+            }
+        """
+        if not self.iswin32:
+            skip("windows specific")
+        from _ffi import CDLL, types
+        libfoo = CDLL(self.libfoo_name)
+        f_name = libfoo.getfunc('AAA_first_ordinal_function', [], types.sint)
+        f_ordinal = libfoo.getfunc(1, [], types.sint)
+        assert f_name.getaddr() == f_ordinal.getaddr()
