@@ -1,8 +1,6 @@
 from __future__ import with_statement
 import py, os, errno
 
-from pypy.conftest import gettestobjspace, option
-
 def getfile(space):
     return space.appexec([], """():
         try:
@@ -13,8 +11,9 @@ def getfile(space):
     """)
 
 class AppTestFile(object):
+    spaceconfig = dict(usemodules=("_file",))
+
     def setup_class(cls):
-        cls.space = gettestobjspace(usemodules=("_file", ))
         cls.w_temppath = cls.space.wrap(
             str(py.test.ensuretemp("fileimpl").join("foo.txt")))
         cls.w_file = getfile(cls.space)
@@ -263,7 +262,7 @@ class AppTestNonblocking(object):
 
         cls.old_read = os.read
 
-        if option.runappdirect:
+        if cls.runappdirect:
             py.test.skip("works with internals of _file impl on py.py")
         state = [0]
         def read(fd, n=None):
@@ -293,10 +292,11 @@ class AppTestConcurrency(object):
     # these tests only really make sense on top of a translated pypy-c,
     # because on top of py.py the inner calls to os.write() don't
     # release our object space's GIL.
+    spaceconfig = dict(usemodules=("_file",))
+
     def setup_class(cls):
-        if not option.runappdirect:
+        if not cls.runappdirect:
             py.test.skip("likely to deadlock when interpreted by py.py")
-        cls.space = gettestobjspace(usemodules=("_file", "thread"))
         cls.w_temppath = cls.space.wrap(
             str(py.test.ensuretemp("fileimpl").join("concurrency.txt")))
         cls.w_file = getfile(cls.space)
@@ -387,8 +387,9 @@ class AppTestConcurrency(object):
 
 
 class AppTestFile25:
+    spaceconfig = dict(usemodules=("_file",))
+
     def setup_class(cls):
-        cls.space = gettestobjspace(usemodules=("_file", ))
         cls.w_temppath = cls.space.wrap(
             str(py.test.ensuretemp("fileimpl").join("foo.txt")))
         cls.w_file = getfile(cls.space)
@@ -428,10 +429,22 @@ class AppTestFile25:
             pass
         assert f.subclass_closed
 
+    def test_readline_unbuffered_should_read_one_line_only(self):
+        import os
+
+        with self.file(self.temppath, 'wb') as f:
+            f.write('foo\nbar\n')
+
+        with self.file(self.temppath, 'rb', 0) as f:
+            s = f.readline()
+            assert s == 'foo\n'
+            s = os.read(f.fileno(), 10)
+            assert s == 'bar\n'
+
 def test_flush_at_exit():
     from pypy import conftest
     from pypy.tool.option import make_config, make_objspace
-    from pypy.tool.udir import udir
+    from rpython.tool.udir import udir
 
     tmpfile = udir.join('test_flush_at_exit')
     config = make_config(conftest.option)
