@@ -1,17 +1,17 @@
-from pypy.module._io.interp_iobase import W_IOBase
-from pypy.interpreter.typedef import (
-    TypeDef, GetSetProperty, interp_attrproperty_w, interp_attrproperty,
-    generic_new_descr)
-from pypy.interpreter.gateway import interp2app, unwrap_spec
-from pypy.interpreter.baseobjspace import Wrappable
-from pypy.interpreter.error import OperationError
-from pypy.rlib.rarithmetic import intmask, r_ulonglong, r_uint
-from pypy.rlib.rbigint import rbigint
-from pypy.rlib.rstring import UnicodeBuilder
-from pypy.module._codecs import interp_codecs
-from pypy.module._io.interp_iobase import convert_size
 import sys
 
+from pypy.interpreter.baseobjspace import W_Root
+from pypy.interpreter.error import OperationError
+from pypy.interpreter.gateway import WrappedDefault, interp2app, unwrap_spec
+from pypy.interpreter.typedef import (
+    GetSetProperty, TypeDef, generic_new_descr, interp_attrproperty,
+    interp_attrproperty_w)
+from pypy.module._codecs import interp_codecs
+from pypy.module._io.interp_iobase import W_IOBase, convert_size
+from rpython.rlib.rarithmetic import intmask, r_uint, r_ulonglong
+from rpython.rlib.rbigint import rbigint
+from rpython.rlib.rstring import UnicodeBuilder
+
 
 STATE_ZERO, STATE_OK, STATE_DETACHED = range(3)
 
@@ -22,7 +22,7 @@ SEEN_ALL  = SEEN_CR | SEEN_LF | SEEN_CRLF
 
 _WINDOWS = sys.platform == 'win32'
 
-class W_IncrementalNewlineDecoder(Wrappable):
+class W_IncrementalNewlineDecoder(W_Root):
     seennl = 0
     pendingcr = False
     w_decoder = None
@@ -46,13 +46,12 @@ class W_IncrementalNewlineDecoder(Wrappable):
     def descr_init(self, space, w_decoder, translate, w_errors=None):
         self.w_decoder = w_decoder
         self.translate = translate
-        if space.is_w(w_errors, space.w_None):
+        if space.is_none(w_errors):
             self.w_errors = space.wrap("strict")
         else:
             self.w_errors = w_errors
 
         self.seennl = 0
-        pendingcr = False
 
     def newlines_get_w(self, space):
         return self.w_newlines_dict.get(self.seennl, space.w_None)
@@ -271,6 +270,27 @@ W_TextIOBase.typedef = TypeDef(
     errors = GetSetProperty(W_TextIOBase.errors_get_w),
 )
 
+
+def _determine_encoding(space, encoding):
+    if encoding is not None:
+        return space.wrap(encoding)
+
+    try:
+        w_locale = space.call_method(space.builtin, '__import__',
+                                     space.wrap('locale'))
+        w_encoding = space.call_method(w_locale, 'getpreferredencoding')
+    except OperationError as e:
+        # getpreferredencoding() may also raise ImportError
+        if not e.match(space, space.w_ImportError):
+            raise
+        return space.wrap('ascii')
+    else:
+        if space.isinstance_w(w_encoding, space.w_str):
+            return w_encoding
+
+    raise OperationError(space.w_IOError, space.wrap(
+            "could not determine default encoding"))
+
 class PositionCookie(object):
     def __init__(self, bigint):
         self.start_pos = bigint.ulonglongmask()
@@ -338,38 +358,14 @@ class W_TextIOWrapper(W_TextIOBase):
     def descr_init(self, space, w_buffer, encoding=None,
                    w_errors=None, w_newline=None, line_buffering=0):
         self.state = STATE_ZERO
-
         self.w_buffer = w_buffer
+        self.w_encoding = _determine_encoding(space, encoding)
 
-        # Set encoding
-        self.w_encoding = None
-        if encoding is None:
-            try:
-                w_locale = space.call_method(space.builtin, '__import__',
-                                             space.wrap("locale"))
-                self.w_encoding = space.call_method(w_locale,
-                                                    "getpreferredencoding")
-            except OperationError, e:
-                # getpreferredencoding() may also raise ImportError
-                if not e.match(space, space.w_ImportError):
-                    raise
-                self.w_encoding = space.wrap("ascii")
-            else:
-                if not space.isinstance_w(self.w_encoding, space.w_str):
-                    self.w_encoding = None
-        if self.w_encoding:
-            pass
-        elif encoding is not None:
-            self.w_encoding = space.wrap(encoding)
-        else:
-            raise OperationError(space.w_IOError, space.wrap(
-                "could not determine default encoding"))
-
-        if space.is_w(w_errors, space.w_None):
+        if space.is_none(w_errors):
             w_errors = space.wrap("strict")
         self.w_errors = w_errors
 
-        if space.is_w(w_newline, space.w_None):
+        if space.is_none(w_newline):
             newline = None
         else:
             newline = space.unicode_w(w_newline)
@@ -484,6 +480,7 @@ class W_TextIOWrapper(W_TextIOBase):
         self._writeflush(space)
         space.call_method(self.w_buffer, "flush")
 
+    @unwrap_spec(w_pos = WrappedDefault(None))
     def truncate_w(self, space, w_pos=None):
         self._check_init(space)
 
