@@ -1,16 +1,15 @@
-
-
-from pypy.conftest import gettestobjspace
-from pypy.translator.platform import platform
-from pypy.translator.tool.cbuild import ExternalCompilationInfo
+from rpython.translator.platform import platform
+from rpython.translator.tool.cbuild import ExternalCompilationInfo
 from pypy.module._rawffi.interp_rawffi import TYPEMAP
 from pypy.module._rawffi.tracker import Tracker
 
 import os, sys, py
 
 class AppTestFfi:
+    spaceconfig = dict(usemodules=['_rawffi', 'struct'])
+
     def prepare_c_example():
-        from pypy.tool.udir import udir
+        from rpython.tool.udir import udir
         c_file = udir.ensure("test__rawffi", dir=1).join("xlib.c")
         c_file.write(py.code.Source('''
         #include <stdlib.h>
@@ -197,9 +196,8 @@ class AppTestFfi:
     prepare_c_example = staticmethod(prepare_c_example)
     
     def setup_class(cls):
-        from pypy.rlib.clibffi import get_libc_name
-        space = gettestobjspace(usemodules=('_rawffi', 'struct'))
-        cls.space = space
+        space = cls.space
+        from rpython.rlib.clibffi import get_libc_name
         cls.w_lib_name = space.wrap(cls.prepare_c_example())
         cls.w_libc_name = space.wrap(get_libc_name())
         if sys.platform == 'win32':
@@ -555,6 +553,7 @@ class AppTestFfi:
         raises(ValueError, "lib.ptr('get_char', [], 'x')")
         raises(ValueError, "_rawffi.Structure(['x1', 'xx'])")
         raises(ValueError, _rawffi.Structure, [('x1', 'xx')])
+        raises(TypeError, _rawffi.Structure, [(u'\xe9', 'l')])
         raises(ValueError, "_rawffi.Array('xx')")
 
     def test_longs_ulongs(self):
@@ -1029,10 +1028,10 @@ class AppTestFfi:
         S2E.get_ffi_type()     # does not hang
 
 class AppTestAutoFree:
+    spaceconfig = dict(usemodules=['_rawffi', 'struct'])
+    
     def setup_class(cls):
-        space = gettestobjspace(usemodules=('_rawffi', 'struct'))
-        cls.space = space
-        cls.w_sizes_and_alignments = space.wrap(dict(
+        cls.w_sizes_and_alignments = cls.space.wrap(dict(
             [(k, (v.c_size, v.c_alignment)) for k,v in TYPEMAP.iteritems()]))
         Tracker.DO_TRACING = True
 
