@@ -1,4 +1,4 @@
-from pypy.interpreter.gateway import unwrap_spec
+from pypy.interpreter.gateway import unwrap_spec, WrappedDefault
 from pypy.interpreter.error import OperationError
 
 def create_filter(space, w_category, action):
@@ -57,7 +57,7 @@ def get_category(space, w_message, w_category):
     # Get category
     if space.isinstance_w(w_message, space.w_Warning):
         w_category = space.type(w_message)
-    elif space.is_w(w_category, space.w_None):
+    elif space.is_none(w_category):
         w_category = space.w_UserWarning
 
     # Validate category
@@ -101,9 +101,8 @@ def setup_context(space, stacklevel):
     # setup filename
     try:
         w_filename = space.getitem(w_globals, space.wrap("__file__"))
+        filename = space.str_w(w_filename)
     except OperationError, e:
-        if not e.match(space, space.w_KeyError):
-            raise
         if space.str_w(w_module) == '__main__':
             w_argv = space.sys.getdictvalue(space, 'argv')
             if w_argv and space.len_w(w_argv) > 0:
@@ -116,12 +115,11 @@ def setup_context(space, stacklevel):
         else:
             w_filename = w_module
     else:
-        # if filename.lower().endswith((".pyc", ".pyo"))
-        if space.is_true(space.call_method(
-            w_filename, "endswith",
-            space.newtuple([space.wrap(".pyc"), space.wrap(".pyo")]))):
+        lc_filename = filename.lower()
+        if (lc_filename.endswith(".pyc") or 
+            lc_filename.endswith(".pyo")):
             # strip last character
-            w_filename = space.wrap(space.str_w(w_filename)[:-1])
+            w_filename = space.wrap(filename[:-1])
 
     return (w_filename, lineno, w_module, w_registry)
 
@@ -192,7 +190,6 @@ def normalize_module(space, w_filename):
         return space.wrap("<unknown>")
 
     filename = space.str_w(w_filename)
-    length = len(filename)
     if filename.endswith(".py"):
         n = len(filename) - 3
         assert n >= 0
@@ -313,7 +310,7 @@ def warn(space, w_message, w_category=None, stacklevel=1):
 
 
 def get_source_line(space, w_globals, lineno):
-    if space.is_w(w_globals, space.w_None):
+    if space.is_none(w_globals):
         return None
 
     # Check/get the requisite pieces needed for the loader.
@@ -345,7 +342,9 @@ def get_source_line(space, w_globals, lineno):
     w_source_line = space.getitem(w_source_list, space.wrap(lineno - 1))
     return w_source_line
 
-@unwrap_spec(lineno=int)
+@unwrap_spec(lineno=int, w_module = WrappedDefault(None),
+             w_registry = WrappedDefault(None),
+             w_module_globals = WrappedDefault(None))
 def warn_explicit(space, w_message, w_category, w_filename, lineno,
                   w_module=None, w_registry=None, w_module_globals=None):
 
