@@ -1,5 +1,6 @@
 from pypy.interpreter.mixedmodule import MixedModule
-from pypy.module.array.interp_array import types, W_ArrayBase
+
+from pypy.module.array.interp_array import types
 from pypy.objspace.std.model import registerimplementation
 
 for mytype in types.values():
@@ -7,7 +8,6 @@ for mytype in types.values():
 
 
 class Module(MixedModule):
-
     interpleveldefs = {
         'array': 'interp_array.W_ArrayBase',
         'ArrayType': 'interp_array.W_ArrayBase',
