@@ -1,12 +1,11 @@
-from pypy.conftest import gettestobjspace
 import sys
 import py
 import py.test
 
 
 ## class AppTestSimpleArray:
+##     spaceconfig = dict(usemodules=('array',))
 ##     def setup_class(cls):
-##         cls.space = gettestobjspace(usemodules=('array',))
 ##         cls.w_simple_array = cls.space.appexec([], """():
 ##             import array
 ##             return array.simple_array
@@ -554,6 +553,15 @@ class BaseArrayTests:
                 assert (a >= c) is False
                 assert (c >= a) is True
 
+        a = self.array('i', [-1, 0, 1, 42, 0x7f])
+        assert not a == 2*a
+        assert a != 2*a
+        assert a < 2*a
+        assert a <= 2*a
+        assert not a > 2*a
+        assert not a >= 2*a
+
+
     def test_reduce(self):
         import pickle
         a = self.array('i', [1, 2, 3])
@@ -681,6 +689,22 @@ class BaseArrayTests:
         a.__delslice__(0, 2)
         assert repr(a) == "array('i', [5])"
 
+        a = self.array('i', [1, 2, 3, 4, 5])
+        del a[3:1]
+        assert repr(a) == "array('i', [1, 2, 3, 4, 5])"
+
+        del a[-100:1]
+        assert repr(a) == "array('i', [2, 3, 4, 5])"
+
+        del a[3:]
+        assert repr(a) == "array('i', [2, 3, 4])"
+
+        del a[-1:]
+        assert repr(a) == "array('i', [2, 3])"
+
+        del a[1:100]
+        assert repr(a) == "array('i', [2])"
+
     def test_iter(self):
         a = self.array('i', [1, 2, 3])
         assert 1 in a
@@ -862,12 +886,11 @@ class TestCPythonsOwnArray(BaseArrayTests):
         cls.tempfile = str(py.test.ensuretemp('array').join('tmpfile'))
         cls.maxint = sys.maxint
 
+
 class AppTestArray(BaseArrayTests):
-    OPTIONS = {}
+    spaceconfig = {'usemodules': ['array', 'struct', '_rawffi', 'binascii']}
 
     def setup_class(cls):
-        cls.space = gettestobjspace(usemodules=('array', 'struct', '_rawffi'),
-                                    **cls.OPTIONS)
         cls.w_array = cls.space.appexec([], """():
             import array
             return array.array
@@ -875,7 +898,7 @@ class AppTestArray(BaseArrayTests):
         cls.w_tempfile = cls.space.wrap(
             str(py.test.ensuretemp('array').join('tmpfile')))
         cls.w_maxint = cls.space.wrap(sys.maxint)
-    
+
     def test_buffer_info(self):
         a = self.array('c', 'Hi!')
         bi = a.buffer_info()
@@ -890,6 +913,56 @@ class AppTestArray(BaseArrayTests):
         a[::-1] = a
         assert a == self.array('b', [3, 2, 1, 0])
 
+    def test_array_multiply(self):
+        a = self.array('b', [0])
+        b = a * 13
+        assert b[12] == 0
+        b = 13 * a
+        assert b[12] == 0
+        a *= 13
+        assert a[12] == 0
+        a = self.array('b', [1])
+        b = a * 13
+        assert b[12] == 1
+        b = 13 * a
+        assert b[12] == 1
+        a *= 13
+        assert a[12] == 1
+        a = self.array('i', [0])
+        b = a * 13
+        assert b[12] == 0
+        b = 13 * a
+        assert b[12] == 0
+        a *= 13
+        assert a[12] == 0
+        a = self.array('i', [1])
+        b = a * 13
+        assert b[12] == 1
+        b = 13 * a
+        assert b[12] == 1
+        a *= 13
+        assert a[12] == 1
+        a = self.array('i', [0, 0])
+        b = a * 13
+        assert len(b) == 26
+        assert b[22] == 0
+        b = 13 * a
+        assert len(b) == 26
+        assert b[22] == 0
+        a *= 13
+        assert a[22] == 0
+        assert len(a) == 26
+        a = self.array('f', [-0.0])
+        b = a * 13
+        assert len(b) == 13
+        assert str(b[12]) == "-0.0"
+        a = self.array('d', [-0.0])
+        b = a * 13
+        assert len(b) == 13
+        assert str(b[12]) == "-0.0"
+
 
 class AppTestArrayBuiltinShortcut(AppTestArray):
-    OPTIONS = {'objspace.std.builtinshortcut': True}
+    spaceconfig = AppTestArray.spaceconfig.copy()
+    spaceconfig['objspace.std.builtinshortcut'] = True
+
