@@ -1,11 +1,12 @@
-from pypy.interpreter.baseobjspace import Wrappable
+from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.typedef import TypeDef, GetSetProperty
 from pypy.interpreter.argument import Arguments
 from pypy.interpreter.typedef import interp_attrproperty, interp_attrproperty_w
 from pypy.interpreter.gateway import interp2app
 from pypy.interpreter.error import OperationError, operationerrfmt
-from pypy.interpreter.function import BuiltinFunction, Method, StaticMethod
-from pypy.rpython.lltypesystem import rffi, lltype
+from pypy.interpreter.function import (
+    BuiltinFunction, Method, StaticMethod, ClassMethod)
+from rpython.rtyper.lltypesystem import rffi, lltype
 from pypy.module.cpyext.pyobject import (PyObject, from_ref, make_ref,
                                          make_typedescr, Py_DecRef)
 from pypy.module.cpyext.api import (
@@ -13,7 +14,7 @@ from pypy.module.cpyext.api import (
     METH_O, CONST_STRING, METH_CLASS, METH_STATIC, METH_COEXIST, METH_NOARGS,
     METH_VARARGS, build_type_checkers, PyObjectFields, bootstrap_function)
 from pypy.module.cpyext.pyerrors import PyErr_Occurred
-from pypy.rlib.objectmodel import we_are_translated
+from rpython.rlib.objectmodel import we_are_translated
 
 PyCFunction_typedef = rffi.COpaquePtr(typedef='PyCFunction')
 PyCFunction = lltype.Ptr(lltype.FuncType([PyObject, PyObject], PyObject))
@@ -58,7 +59,8 @@ def cfunction_dealloc(space, py_obj):
     from pypy.module.cpyext.object import PyObject_dealloc
     PyObject_dealloc(space, py_obj)
 
-class W_PyCFunctionObject(Wrappable):
+
+class W_PyCFunctionObject(W_Root):
     def __init__(self, space, ml, w_self, w_module=None):
         self.ml = ml
         self.name = rffi.charp2str(self.ml.c_ml_name)
@@ -128,7 +130,22 @@ class W_PyCMethodObject(W_PyCFunctionObject):
 
 PyCFunction_Check, PyCFunction_CheckExact = build_type_checkers("CFunction", W_PyCFunctionObject)
 
-class W_PyCWrapperObject(Wrappable):
+class W_PyCClassMethodObject(W_PyCFunctionObject):
+    w_self = None
+    def __init__(self, space, ml, w_type):
+        self.space = space
+        self.ml = ml
+        self.name = rffi.charp2str(ml.c_ml_name)
+        self.w_objclass = w_type
+
+    def __repr__(self):
+        return self.space.unwrap(self.descr_method_repr())
+
+    def descr_method_repr(self):
+        return self.getrepr(self.space, "built-in method '%s' of '%s' object" % (self.name, self.w_objclass.getname(self.space)))
+
+
+class W_PyCWrapperObject(W_Root):
     def __init__(self, space, pto, method_name, wrapper_func, wrapper_func_kwds,
             doc, func):
         self.space = space
@@ -188,7 +205,7 @@ def cmethod_descr_call(space, w_self, __args__):
     return ret
 
 def cmethod_descr_get(space, w_function, w_obj, w_cls=None):
-    asking_for_bound = (space.is_w(w_cls, space.w_None) or
+    asking_for_bound = (space.is_none(w_cls) or
                         not space.is_w(w_obj, space.w_None) or
                         space.is_w(w_cls, space.type(space.w_None)))
     if asking_for_bound:
@@ -196,6 +213,11 @@ def cmethod_descr_get(space, w_function, w_obj, w_cls=None):
     else:
         return w_function
 
+def cclassmethod_descr_get(space, w_function, w_obj, w_cls=None):
+    if not w_cls:
+        w_cls = space.type(w_obj)
+    return space.wrap(Method(space, w_function, w_cls, space.w_None))
+
 
 W_PyCFunctionObject.typedef = TypeDef(
     'builtin_function_or_method',
@@ -216,6 +238,16 @@ W_PyCMethodObject.typedef = TypeDef(
     )
 W_PyCMethodObject.typedef.acceptable_as_base_class = False
 
+W_PyCClassMethodObject.typedef = TypeDef(
+    'classmethod',
+    __get__ = interp2app(cclassmethod_descr_get),
+    __call__ = interp2app(cmethod_descr_call),
+    __name__ = interp_attrproperty('name', cls=W_PyCClassMethodObject),
+    __objclass__ = interp_attrproperty_w('w_objclass', cls=W_PyCClassMethodObject),
+    __repr__ = interp2app(W_PyCClassMethodObject.descr_method_repr),
+    )
+W_PyCClassMethodObject.typedef.acceptable_as_base_class = False
+
 
 W_PyCWrapperObject.typedef = TypeDef(
     'wrapper_descriptor',
@@ -243,10 +275,18 @@ def PyCFunction_GetFunction(space, w_obj):
 def PyStaticMethod_New(space, w_func):
     return space.wrap(StaticMethod(w_func))
 
+@cpython_api([PyObject], PyObject)
+def PyClassMethod_New(space, w_func):
+    return space.wrap(ClassMethod(w_func))
+
 @cpython_api([PyObject, lltype.Ptr(PyMethodDef)], PyObject)
 def PyDescr_NewMethod(space, w_type, method):
     return space.wrap(W_PyCMethodObject(space, method, w_type))
 
+@cpython_api([PyObject, lltype.Ptr(PyMethodDef)], PyObject)
+def PyDescr_NewClassMethod(space, w_type, method):
+    return space.wrap(W_PyCClassMethodObject(space, method, w_type))
+
 def PyDescr_NewWrapper(space, pto, method_name, wrapper_func, wrapper_func_kwds,
                        doc, func):
     # not exactly the API sig
