@@ -1,6 +1,5 @@
 from pypy.interpreter.error import OperationError
-from pypy.rpython.lltypesystem import rffi, lltype
-from pypy.rpython.lltypesystem import llmemory
+from rpython.rtyper.lltypesystem import rffi, lltype
 from pypy.module.unicodedata import unicodedb
 from pypy.module.cpyext.api import (
     CANNOT_FAIL, Py_ssize_t, build_type_checkers, cpython_api,
@@ -12,9 +11,10 @@ from pypy.module.cpyext.pyobject import (
     make_typedescr, get_typedescr)
 from pypy.module.cpyext.stringobject import PyString_Check
 from pypy.module.sys.interp_encoding import setdefaultencoding
+from pypy.module._codecs.interp_codecs import CodecState
 from pypy.objspace.std import unicodeobject, unicodetype, stringtype
-from pypy.rlib import runicode
-from pypy.tool.sourcetools import func_renamer
+from rpython.rlib import runicode
+from rpython.tool.sourcetools import func_renamer
 import sys
 
 ## See comment in stringobject.py.
@@ -387,7 +387,7 @@ def PyUnicode_FromEncodedObject(space, w_obj, encoding, errors):
 
     # - unicode is disallowed
     # - raise TypeError for non-string types
-    if space.is_true(space.isinstance(w_obj, space.w_unicode)):
+    if space.isinstance_w(w_obj, space.w_unicode):
         w_meth = None
     else:
         try:
@@ -610,6 +610,38 @@ def PyUnicode_DecodeUTF32(space, s, size, llerrors, pbyteorder):
 
     return space.wrap(result)
 
+@cpython_api([rffi.CWCHARP, Py_ssize_t, rffi.CCHARP, rffi.CCHARP],
+             rffi.INT_real, error=-1)
+def PyUnicode_EncodeDecimal(space, s, length, output, llerrors):
+    """Takes a Unicode string holding a decimal value and writes it
+    into an output buffer using standard ASCII digit codes.
+
+    The output buffer has to provide at least length+1 bytes of
+    storage area. The output string is 0-terminated.
+
+    The encoder converts whitespace to ' ', decimal characters to
+    their corresponding ASCII digit and all other Latin-1 characters
+    except \0 as-is. Characters outside this range (Unicode ordinals
+    1-256) are treated as errors. This includes embedded NULL bytes.
+
+    Returns 0 on success, -1 on failure.
+    """
+    u = rffi.wcharpsize2unicode(s, length)
+    if llerrors:
+        errors = rffi.charp2str(llerrors)
+    else:
+        errors = None
+    state = space.fromcache(CodecState)
+    result = runicode.unicode_encode_decimal(u, length, errors,
+                                             state.encode_error_handler)
+    i = len(result)
+    output[i] = '\0'
+    i -= 1
+    while i >= 0:
+        output[i] = result[i]
+        i -= 1
+    return 0
+
 @cpython_api([PyObject, PyObject], rffi.INT_real, error=-2)
 def PyUnicode_Compare(space, w_left, w_right):
     """Compare two strings and return -1, 0, 1 for less than, equal, and greater
