@@ -1,5 +1,5 @@
 
-from pypy.rlib.objectmodel import specialize
+from rpython.rlib.objectmodel import specialize
 
 class AppBridgeCache(object):
     w__var = None
