@@ -1,24 +1,51 @@
 
 import sys
-from pypy.interpreter.baseobjspace import Wrappable
+from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.error import OperationError
 from pypy.interpreter.gateway import interp2app, unwrap_spec
 from pypy.interpreter.typedef import (TypeDef, GetSetProperty,
     interp_attrproperty, interp_attrproperty_w)
-from pypy.module.micronumpy import types, interp_boxes
-from pypy.rlib.objectmodel import specialize
-from pypy.rlib.rarithmetic import LONG_BIT, r_longlong, r_ulonglong
+from pypy.module.micronumpy import types, interp_boxes, base
+from rpython.rlib.objectmodel import specialize
+from rpython.rlib.rarithmetic import LONG_BIT, r_longlong, r_ulonglong
+from rpython.rtyper.lltypesystem import rffi
+from rpython.rlib import jit
 
 
 UNSIGNEDLTR = "u"
 SIGNEDLTR = "i"
 BOOLLTR = "b"
 FLOATINGLTR = "f"
+COMPLEXLTR = "c"
 VOIDLTR = 'V'
 STRINGLTR = 'S'
 UNICODELTR = 'U'
+INTPLTR = 'p'
+UINTPLTR = 'P'
 
-class W_Dtype(Wrappable):
+def decode_w_dtype(space, w_dtype):
+    if space.is_none(w_dtype):
+        return None
+    return space.interp_w(W_Dtype,
+          space.call_function(space.gettypefor(W_Dtype), w_dtype))
+
+@jit.unroll_safe
+def dtype_agreement(space, w_arr_list, shape, out=None):
+    """ agree on dtype from a list of arrays. if out is allocated,
+    use it's dtype, otherwise allocate a new one with agreed dtype
+    """
+    from pypy.module.micronumpy.interp_ufuncs import find_binop_result_dtype
+
+    if not space.is_none(out):
+        return out
+    dtype = w_arr_list[0].get_dtype()
+    for w_arr in w_arr_list[1:]:
+        dtype = find_binop_result_dtype(space, dtype, w_arr.get_dtype())
+    out = base.W_NDimArray.from_shape(shape, dtype)
+    return out
+
+
+class W_Dtype(W_Root):
     _immutable_fields_ = ["itemtype", "num", "kind"]
 
     def __init__(self, itemtype, num, kind, name, char, w_box_type,
@@ -35,31 +62,43 @@ class W_Dtype(Wrappable):
         self.fields = fields
         self.fieldnames = fieldnames
         self.native = native
+        self.float_type = None
 
     @specialize.argtype(1)
     def box(self, value):
         return self.itemtype.box(value)
 
+    @specialize.argtype(1, 2)
+    def box_complex(self, real, imag):
+        return self.itemtype.box_complex(real, imag)
+
+    def build_and_convert(self, space, box):
+        return self.itemtype.build_and_convert(space, self, box)
     def coerce(self, space, w_item):
         return self.itemtype.coerce(space, self, w_item)
 
     def getitem(self, arr, i):
-        return self.itemtype.read(arr, 1, i, 0)
+        return self.itemtype.read(arr, i, 0)
 
     def getitem_bool(self, arr, i):
-        return self.itemtype.read_bool(arr, 1, i, 0)
+        return self.itemtype.read_bool(arr, i, 0)
 
     def setitem(self, arr, i, box):
-        self.itemtype.store(arr, 1, i, 0, box)
+        self.itemtype.store(arr, i, 0, box)
 
     def fill(self, storage, box, start, stop):
         self.itemtype.fill(storage, self.get_size(), box, start, stop, 0)
 
+    def get_name(self):
+        if self.char == 'S':
+            return '|S' + str(self.get_size())
+        return self.name
+
     def descr_str(self, space):
-        return space.wrap(self.name)
+        return space.wrap(self.get_name())
 
     def descr_repr(self, space):
-        return space.wrap("dtype('%s')" % self.name)
+        return space.wrap("dtype('%s')" % self.get_name())
 
     def descr_get_itemsize(self, space):
         return space.wrap(self.itemtype.get_element_size())
@@ -115,12 +154,24 @@ class W_Dtype(Wrappable):
     def is_signed(self):
         return self.kind == SIGNEDLTR
 
+    def is_complex_type(self):
+        return False
+
+    def is_float_type(self):
+        return (self.kind == FLOATINGLTR or self.float_type is not None)
+
     def is_bool_type(self):
         return self.kind == BOOLLTR
 
     def is_record_type(self):
         return self.fields is not None
 
+    def is_str_or_unicode(self):
+        return (self.num == 18 or self.num == 19)
+
+    def is_flexible_type(self):
+        return (self.is_str_or_unicode() or self.is_record_type())
+
     def __repr__(self):
         if self.fields is not None:
             return '<DType %r>' % self.fields
@@ -129,6 +180,18 @@ class W_Dtype(Wrappable):
     def get_size(self):
         return self.itemtype.get_element_size()
 
+class W_ComplexDtype(W_Dtype):
+    def __init__(self, itemtype, num, kind, name, char, w_box_type,
+                 alternate_constructors=[], aliases=[],
+                 fields=None, fieldnames=None, native=True, float_type=None):
+        W_Dtype.__init__(self, itemtype, num, kind, name, char, w_box_type,
+                 alternate_constructors=alternate_constructors, aliases=aliases,
+                 fields=fields, fieldnames=fieldnames, native=native)
+        self.float_type = float_type
+
+    def is_complex_type(self):
+        return True
+
 def dtype_from_list(space, w_lst):
     lst_w = space.listview(w_lst)
     fields = {}
@@ -166,7 +229,7 @@ def variable_dtype(space, name):
             size = int(name[1:])
         except ValueError:
             raise OperationError(space.w_TypeError, space.wrap("data type not understood"))
-    if char == 'S':
+    if char == 'S' or char == 'c':
         itemtype = types.StringType(size)
         basename = 'string'
         num = 18
@@ -194,7 +257,7 @@ def dtype_from_spec(space, name):
 def descr__new__(space, w_subtype, w_dtype):
     cache = get_dtype_cache(space)
 
-    if space.is_w(w_dtype, space.w_None):
+    if space.is_none(w_dtype):
         return cache.w_float64dtype
     elif space.isinstance_w(w_dtype, w_subtype):
         return w_dtype
@@ -206,19 +269,23 @@ def descr__new__(space, w_subtype, w_dtype):
             return cache.dtypes_by_name[name]
         except KeyError:
             pass
-        if name[0] in 'VSU' or name[0] in '<>=' and name[1] in 'VSU':
+        if name[0] in 'VSUc' or name[0] in '<>=' and name[1] in 'VSUc':
             return variable_dtype(space, name)
+        raise OperationError(space.w_TypeError, space.wrap(
+                       "data type %s not understood" % name))
     elif space.isinstance_w(w_dtype, space.w_list):
         return dtype_from_list(space, w_dtype)
     elif space.isinstance_w(w_dtype, space.w_dict):
         return dtype_from_dict(space, w_dtype)
-    else:
-        for dtype in cache.builtin_dtypes:
-            if w_dtype in dtype.alternate_constructors:
-                return dtype
-            if w_dtype is dtype.w_box_type:
-                return dtype
-    raise OperationError(space.w_TypeError, space.wrap("data type not understood"))
+    for dtype in cache.builtin_dtypes:
+        if w_dtype in dtype.alternate_constructors:
+            return dtype
+        if w_dtype is dtype.w_box_type:
+            return dtype
+    typename = space.type(w_dtype).getname(space)
+    raise OperationError(space.w_TypeError, space.wrap(
+                             "data type not understood (value of type " +
+                             "%s not expected here)" % typename))
 
 W_Dtype.typedef = TypeDef("dtype",
     __module__ = "numpypy",
@@ -342,7 +409,11 @@ class DtypeCache(object):
             name=name,
             char="l",
             w_box_type=space.gettypefor(interp_boxes.W_LongBox),
-            alternate_constructors=[space.w_int],
+            alternate_constructors=[space.w_int,
+                                    space.gettypefor(interp_boxes.W_IntegerBox),
+                                    space.gettypefor(interp_boxes.W_SignedIntegerBox),
+                                   ],
+            aliases=['int'],
         )
         self.w_ulongdtype = W_Dtype(
             types.ULong(),
@@ -351,6 +422,9 @@ class DtypeCache(object):
             name="u" + name,
             char="L",
             w_box_type=space.gettypefor(interp_boxes.W_ULongBox),
+            alternate_constructors=[ space.gettypefor(interp_boxes.W_UnsignedIntegerBox),
+                                   ],
+            aliases=['uint'],
         )
         self.w_int64dtype = W_Dtype(
             types.Int64(),
@@ -384,11 +458,84 @@ class DtypeCache(object):
             name="float64",
             char="d",
             w_box_type = space.gettypefor(interp_boxes.W_Float64Box),
-            alternate_constructors=[space.w_float],
+            alternate_constructors=[space.w_float,
+                                    space.gettypefor(interp_boxes.W_NumberBox),
+                                   ],
             aliases=["float"],
         )
+        self.w_complex64dtype = W_ComplexDtype(
+            types.Complex64(),
+            num=14,
+            kind=COMPLEXLTR,
+            name="complex64",
+            char="F",
+            w_box_type = space.gettypefor(interp_boxes.W_Complex64Box),
+            float_type = self.w_float32dtype,
+        )
+        self.w_complex128dtype = W_ComplexDtype(
+            types.Complex128(),
+            num=15,
+            kind=COMPLEXLTR,
+            name="complex128",
+            char="D",
+            w_box_type = space.gettypefor(interp_boxes.W_Complex128Box),
+            alternate_constructors=[space.w_complex],
+            aliases=["complex"],
+            float_type = self.w_float64dtype,
+        )
+        if interp_boxes.ENABLED_LONG_DOUBLE and interp_boxes.long_double_size == 12:
+            self.w_float96dtype = W_Dtype(
+                types.Float96(),
+                num=13,
+                kind=FLOATINGLTR,
+                name="float96",
+                char="g",
+                w_box_type=space.gettypefor(interp_boxes.W_Float96Box),
+                aliases=["longdouble", "longfloat"],
+            )
+            self.w_complex192dtype = W_ComplexDtype(
+                types.Complex192(),
+                num=16,
+                kind=COMPLEXLTR,
+                name="complex192",
+                char="G",
+                w_box_type = space.gettypefor(interp_boxes.W_Complex192Box),
+                alternate_constructors=[space.w_complex],
+                aliases=["clongdouble", "clongfloat"],
+                float_type = self.w_float96dtype,
+            )
+            self.w_longdouble = self.w_float96dtype
+            self.w_clongdouble = self.w_complex192dtype
+        elif interp_boxes.ENABLED_LONG_DOUBLE and interp_boxes.long_double_size == 16:
+            self.w_float128dtype = W_Dtype(
+                types.Float128(),
+                num=13,
+                kind=FLOATINGLTR,
+                name="float128",
+                char="g",
+                w_box_type=space.gettypefor(interp_boxes.W_Float128Box),
+                aliases=["longdouble", "longfloat"],
+            )
+            self.w_complex256dtype = W_ComplexDtype(
+                types.Complex256(),
+                num=16,
+                kind=COMPLEXLTR,
+                name="complex256",
+                char="G",
+                w_box_type = space.gettypefor(interp_boxes.W_Complex256Box),
+                alternate_constructors=[space.w_complex],
+                aliases=["clongdouble", "clongfloat"],
+                float_type = self.w_float128dtype,
+            )
+            self.w_longdouble = self.w_float128dtype
+            self.w_clongdouble = self.w_complex256dtype
+        elif interp_boxes.ENABLED_LONG_DOUBLE:
+            self.w_float64dtype.aliases += ["longdouble", "longfloat"]
+            self.w_complex128dtype.aliases += ["clongdouble", "clongfloat"]
+            self.w_longdouble = self.w_float64dtype
+            self.w_clongdouble = self.w_complex128dtype
         self.w_stringdtype = W_Dtype(
-            types.StringType(1),
+            types.StringType(0),
             num=18,
             kind=STRINGLTR,
             name='string',
@@ -414,24 +561,78 @@ class DtypeCache(object):
             w_box_type = space.gettypefor(interp_boxes.W_VoidBox),
             #alternate_constructors=[space.w_buffer],
             # XXX no buffer in space
+            #alternate_constructors=[space.gettypefor(interp_boxes.W_GenericBox)],
+            # XXX fix, leads to _coerce error
         )
+        self.w_float16dtype = W_Dtype(
+            types.Float16(),
+            num=23,
+            kind=FLOATINGLTR,
+            name="float16",
+            char="e",
+            w_box_type=space.gettypefor(interp_boxes.W_Float16Box),
+        )
+        ptr_size = rffi.sizeof(rffi.CCHARP)
+        if ptr_size == 4:
+            intp_box = interp_boxes.W_Int32Box
+            intp_type = types.Int32()
+            intp_num = 5
+            uintp_box = interp_boxes.W_UInt32Box
+            uintp_type = types.UInt32()
+            uintp_num = 6
+        elif ptr_size == 8:
+            intp_box = interp_boxes.W_Int64Box
+            intp_type = types.Int64()
+            intp_num = 7
+            uintp_box = interp_boxes.W_UInt64Box
+            uintp_type = types.UInt64()
+            uintp_num = 8
+        else:
+            raise ValueError('unknown point size %d' % ptr_size)
+        self.w_intpdtype = W_Dtype(
+            intp_type,
+            num=intp_num,
+            kind=INTPLTR,
+            name='intp',
+            char=INTPLTR,
+            w_box_type = space.gettypefor(intp_box),
+        )
+        self.w_uintpdtype = W_Dtype(
+            uintp_type,
+            num=uintp_num,
+            kind=UINTPLTR,
+            name='uintp',
+            char=UINTPLTR,
+            w_box_type = space.gettypefor(uintp_box),
+        )
+        float_dtypes = [self.w_float16dtype,
+                self.w_float32dtype, self.w_float64dtype,
+                ]
+        complex_dtypes =  [self.w_complex64dtype, self.w_complex128dtype]
+        if interp_boxes.ENABLED_LONG_DOUBLE:
+            float_dtypes.append(self.w_longdouble)
+            complex_dtypes.append(self.w_clongdouble)
         self.builtin_dtypes = [
-            self.w_booldtype, self.w_int8dtype, self.w_uint8dtype,
-            self.w_int16dtype, self.w_uint16dtype, self.w_int32dtype,
-            self.w_uint32dtype, self.w_longdtype, self.w_ulongdtype,
-            self.w_int64dtype, self.w_uint64dtype,
-            self.w_float32dtype,
-            self.w_float64dtype, self.w_stringdtype, self.w_unicodedtype,
-            self.w_voiddtype,
+            self.w_booldtype,
+            self.w_int8dtype, self.w_uint8dtype,
+            self.w_int16dtype, self.w_uint16dtype,
+            self.w_longdtype, self.w_ulongdtype,
+            self.w_int32dtype, self.w_uint32dtype,
+            self.w_int64dtype, self.w_uint64dtype] + \
+            float_dtypes + complex_dtypes + [
+            self.w_stringdtype, self.w_unicodedtype, self.w_voiddtype,
+            self.w_intpdtype, self.w_uintpdtype,
         ]
         self.float_dtypes_by_num_bytes = sorted(
             (dtype.itemtype.get_element_size(), dtype)
-            for dtype in [self.w_float32dtype, self.w_float64dtype]
+            for dtype in float_dtypes
         )
+        self.dtypes_by_num = {}
         self.dtypes_by_name = {}
         # we reverse, so the stuff with lower numbers override stuff with
         # higher numbers
         for dtype in reversed(self.builtin_dtypes):
+            self.dtypes_by_num[dtype.num] = dtype
             self.dtypes_by_name[dtype.name] = dtype
             can_name = dtype.kind + str(dtype.itemtype.get_element_size())
             self.dtypes_by_name[can_name] = dtype
@@ -444,15 +645,29 @@ class DtypeCache(object):
                 itemtype,
                 dtype.num, dtype.kind, new_name, dtype.char, dtype.w_box_type,
                 native=False)
+            if dtype.kind != dtype.char:
+                can_name = dtype.char
+                self.dtypes_by_name[byteorder_prefix + can_name] = dtype
+                self.dtypes_by_name['=' + can_name] = dtype
+                new_name = nonnative_byteorder_prefix + can_name
+                self.dtypes_by_name[new_name] = W_Dtype(
+                    itemtype,
+                    dtype.num, dtype.kind, new_name, dtype.char, dtype.w_box_type,
+                    native=False)
+
             for alias in dtype.aliases:
                 self.dtypes_by_name[alias] = dtype
             self.dtypes_by_name[dtype.char] = dtype
 
+        self.dtypes_by_num = [dtype for dtype in
+                sorted(self.dtypes_by_num.values(), key=lambda dtype: dtype.num)
+                if dtype.num <= self.w_float64dtype.num]
+        assert len(self.dtypes_by_num) == self.w_float64dtype.num + 1
+
         typeinfo_full = {
             'LONGLONG': self.w_int64dtype,
             'SHORT': self.w_int16dtype,
             'VOID': self.w_voiddtype,
-            #'LONGDOUBLE':,
             'UBYTE': self.w_uint8dtype,
             'UINTP': self.w_ulongdtype,
             'ULONG': self.w_ulongdtype,
@@ -461,10 +676,11 @@ class DtypeCache(object):
             #'OBJECT',
             'ULONGLONG': self.w_uint64dtype,
             'STRING': self.w_stringdtype,
-            #'CDOUBLE',
+            'CDOUBLE': self.w_complex64dtype,
             #'DATETIME',
             'UINT': self.w_uint32dtype,
-            'INTP': self.w_longdtype,
+            'INTP': self.w_intpdtype,
+            'UINTP': self.w_uintpdtype,
             #'HALF',
             'BYTE': self.w_int8dtype,
             #'CFLOAT': ,
@@ -474,8 +690,11 @@ class DtypeCache(object):
             'USHORT': self.w_uint16dtype,
             'FLOAT': self.w_float32dtype,
             'BOOL': self.w_booldtype,
-            #, 'CLONGDOUBLE']
         }
+        if interp_boxes.ENABLED_LONG_DOUBLE:
+            typeinfo_full['LONGDOUBLE'] = self.w_longdouble
+            typeinfo_full['CLONGDOUBLE'] = self.w_clongdouble
+
         typeinfo_partial = {
             'Generic': interp_boxes.W_GenericBox,
             'Character': interp_boxes.W_CharacterBox,
@@ -484,7 +703,7 @@ class DtypeCache(object):
             'Integer': interp_boxes.W_IntegerBox,
             'SignedInteger': interp_boxes.W_SignedIntegerBox,
             'UnsignedInteger': interp_boxes.W_UnsignedIntegerBox,
-            #'ComplexFloating',
+            'ComplexFloating': interp_boxes.W_ComplexFloatingBox,
             'Number': interp_boxes.W_NumberBox,
             'Floating': interp_boxes.W_FloatingBox
         }
@@ -511,9 +730,7 @@ class DtypeCache(object):
                     w_minobj = space.wrap(0)
                 items_w = items_w + [w_maxobj, w_minobj]
             items_w = items_w + [dtype.w_box_type]
-                       
-            w_tuple = space.newtuple(items_w)
-            space.setitem(w_typeinfo, space.wrap(k), w_tuple)
+            space.setitem(w_typeinfo, space.wrap(k), space.newtuple(items_w))
         self.w_typeinfo = w_typeinfo
 
 def get_dtype_cache(space):
