@@ -1,16 +1,18 @@
-
+from pypy.conftest import option
+from pypy.interpreter.gateway import interp2app
 from pypy.module.micronumpy.test.test_base import BaseNumpyAppTest
 
+
 class AppTestUfuncs(BaseNumpyAppTest):
     def test_ufunc_instance(self):
-        from _numpypy import add, ufunc
+        from numpypy import add, ufunc
 
         assert isinstance(add, ufunc)
         assert repr(add) == "<ufunc 'add'>"
-        assert repr(ufunc) == "<type 'numpypy.ufunc'>"
+        assert repr(ufunc) == "<type 'numpypy.ufunc'>" or repr(ufunc) == "<type 'numpy.ufunc'>"
 
     def test_ufunc_attrs(self):
-        from _numpypy import add, multiply, sin
+        from numpypy import add, multiply, sin
 
         assert add.identity == 0
         assert multiply.identity == 1
@@ -21,7 +23,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert sin.nin == 1
 
     def test_wrong_arguments(self):
-        from _numpypy import add, sin
+        from numpypy import add, sin
 
         raises(ValueError, add, 1)
         raises(TypeError, add, 1, 2, 3)
@@ -29,14 +31,14 @@ class AppTestUfuncs(BaseNumpyAppTest):
         raises(ValueError, sin)
 
     def test_single_item(self):
-        from _numpypy import negative, sign, minimum
+        from numpypy import negative, sign, minimum
 
         assert negative(5.0) == -5.0
         assert sign(-0.0) == 0.0
         assert minimum(2.0, 3.0) == 2.0
 
     def test_sequence(self):
-        from _numpypy import array, ndarray, negative, minimum
+        from numpypy import array, ndarray, negative, minimum
         a = array(range(3))
         b = [2.0, 1.0, 0.0]
         c = 1.0
@@ -69,8 +71,44 @@ class AppTestUfuncs(BaseNumpyAppTest):
         for i in range(3):
             assert min_c_b[i] == min(b[i], c)
 
+    def test_scalar(self):
+        # tests that by calling all available ufuncs on scalars, none will
+        # raise uncaught interp-level exceptions, (and crash the test)
+        # and those that are uncallable can be accounted for.
+        # test on the four base-class dtypes: int, bool, float, complex
+        # We need this test since they have no common base class.
+        import numpypy as np
+        def find_uncallable_ufuncs(dtype):
+            uncallable = set()
+            array = np.array(1, dtype)
+            for s in dir(np):
+                u = getattr(np, s)
+                if isinstance(u, np.ufunc):
+                    try:
+                        u(* [array] * u.nin)
+                    except TypeError:
+                        assert s not in uncallable
+                        uncallable.add(s)
+            return uncallable
+        assert find_uncallable_ufuncs('int') == set()
+        assert find_uncallable_ufuncs('bool') == set()
+        assert find_uncallable_ufuncs('float') == set(
+                ['bitwise_and', 'bitwise_not', 'bitwise_or', 'bitwise_xor',
+                 'left_shift', 'right_shift', 'invert'])
+        assert find_uncallable_ufuncs('complex') == set(
+                ['bitwise_and', 'bitwise_not', 'bitwise_or', 'bitwise_xor',
+                 'arctan2', 'deg2rad', 'degrees', 'rad2deg', 'radians',
+                 'fabs', 'fmod', 'invert', 'isneginf', 'isposinf',
+                 'logaddexp', 'logaddexp2', 'left_shift', 'right_shift',
+                 'copysign', 'signbit', 'ceil', 'floor', 'trunc'])
+
+    def test_int_only(self):
+        from numpypy import bitwise_and, array
+        a = array(1.0)
+        raises(TypeError, bitwise_and, a, a)
+
     def test_negative(self):
-        from _numpypy import array, negative
+        from numpypy import array, negative
 
         a = array([-5.0, 0.0, 1.0])
         b = negative(a)
@@ -85,7 +123,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert negative(a + a)[3] == -6
 
     def test_abs(self):
-        from _numpypy import array, absolute
+        from numpypy import array, absolute
 
         a = array([-5.0, -0.0, 1.0])
         b = absolute(a)
@@ -93,7 +131,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
             assert b[i] == abs(a[i])
 
     def test_add(self):
-        from _numpypy import array, add
+        from numpypy import array, add
 
         a = array([-5.0, -0.0, 1.0])
         b = array([ 3.0, -2.0,-3.0])
@@ -102,7 +140,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
             assert c[i] == a[i] + b[i]
 
     def test_divide(self):
-        from _numpypy import array, divide
+        from numpypy import array, divide
 
         a = array([-5.0, -0.0, 1.0])
         b = array([ 3.0, -2.0,-3.0])
@@ -113,7 +151,9 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert (divide(array([-10]), array([2])) == array([-5])).all()
 
     def test_true_divide(self):
-        from _numpypy import array, true_divide
+        import math
+        from numpypy import array, true_divide
+        import math
 
         a = array([0, 1, 2, 3, 4, 1, -1])
         b = array([4, 4, 4, 4, 4, 0,  0])
@@ -123,7 +163,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert math.isnan(true_divide(0, 0))
 
     def test_fabs(self):
-        from _numpypy import array, fabs
+        from numpypy import array, fabs, complex128
         from math import fabs as math_fabs, isnan
 
         a = array([-5.0, -0.0, 1.0])
@@ -134,8 +174,9 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert fabs(float('-inf')) == float('inf')
         assert isnan(fabs(float('nan')))
 
+
     def test_fmax(self):
-        from _numpypy import fmax
+        from numpypy import fmax, array
         import math
 
         nnan, nan, inf, ninf = float('-nan'), float('nan'), float('inf'), float('-inf')
@@ -144,8 +185,8 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert (fmax(a, [ninf]*5) == a).all()
         assert (fmax(a, [inf]*5) == [inf]*5).all()
         assert (fmax(a, [1]*5) == [1, 1, 1, 5, inf]).all()
-        assert math.isnan(fmax(nan, 0))
-        assert math.isnan(fmax(0, nan))
+        assert fmax(nan, 0) == 0
+        assert fmax(0, nan) == 0
         assert math.isnan(fmax(nan, nan))
         # The numpy docs specify that the FIRST NaN should be used if both are NaN
         # Since comparisons with nnan and nan all return false,
@@ -155,7 +196,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
 
     def test_fmin(self):
-        from _numpypy import fmin
+        from numpypy import fmin, array
         import math
 
         nnan, nan, inf, ninf = float('-nan'), float('nan'), float('inf'), float('-inf')
@@ -164,16 +205,17 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert (fmin(a, [ninf]*5) == [ninf]*5).all()
         assert (fmin(a, [inf]*5) == a).all()
         assert (fmin(a, [1]*5) == [ninf, -5, 0, 1, 1]).all()
-        assert math.isnan(fmin(nan, 0))
-        assert math.isnan(fmin(0, nan))
+        assert fmin(nan, 0) == 0
+        assert fmin(0, nan) == 0
         assert math.isnan(fmin(nan, nan))
         # The numpy docs specify that the FIRST NaN should be used if both are NaN
         # use copysign on both sides to sidestep bug in nan representaion
         # on Microsoft win32
         assert math.copysign(1., fmin(nnan, nan)) == math.copysign(1., nnan)
 
+
     def test_fmod(self):
-        from _numpypy import fmod
+        from numpypy import fmod
         import math
 
         assert fmod(-1e-100, 1e100) == -1e-100
@@ -183,7 +225,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
             assert math.isnan(fmod(v, 2))
 
     def test_minimum(self):
-        from _numpypy import array, minimum
+        from numpypy import array, minimum
 
         a = array([-5.0, -0.0, 1.0])
         b = array([ 3.0, -2.0,-3.0])
@@ -192,7 +234,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
             assert c[i] == min(a[i], b[i])
 
     def test_maximum(self):
-        from _numpypy import array, maximum
+        from numpypy import array, maximum
 
         a = array([-5.0, -0.0, 1.0])
         b = array([ 3.0, -2.0,-3.0])
@@ -205,7 +247,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert isinstance(x, (int, long))
 
     def test_multiply(self):
-        from _numpypy import array, multiply
+        from numpypy import array, multiply
 
         a = array([-5.0, -0.0, 1.0])
         b = array([ 3.0, -2.0,-3.0])
@@ -214,7 +256,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
             assert c[i] == a[i] * b[i]
 
     def test_sign(self):
-        from _numpypy import array, sign, dtype
+        from numpypy import array, sign, dtype
 
         reference = [-1.0, 0.0, 0.0, 1.0]
         a = array([-5.0, -0.0, 0.0, 6.0])
@@ -227,33 +269,52 @@ class AppTestUfuncs(BaseNumpyAppTest):
         for i in range(10):
             assert a[i] == ref[i]
 
-        a = sign(array([True, False], dtype=bool))
-        assert a.dtype == dtype("int8")
-        assert a[0] == 1
-        assert a[1] == 0
+        a = sign(array([10+10j, -10+10j, 0+10j, 0-10j, 0+0j, 0-0j], dtype=complex))
+        ref = [1, -1, 1, -1, 0, 0]
+        assert (a == ref).all()
 
     def test_signbit(self):
-        from _numpypy import signbit
+        from numpypy import signbit, add
 
         assert (signbit([0, 0.0, 1, 1.0, float('inf')]) ==
             [False, False, False, False, False]).all()
         assert (signbit([-0, -0.0, -1, -1.0, float('-inf')]) ==
             [False,  True,  True,  True,  True]).all()
+
+        a = add.identity
+        assert signbit(a) == False
+
         skip('sign of nan is non-determinant')
         assert (signbit([float('nan'), float('-nan'), -float('nan')]) ==
             [False, True, True]).all()    
 
-    def test_reciporocal(self):
-        from _numpypy import array, reciprocal
+    def test_reciprocal(self):
+        from numpypy import array, reciprocal, complex64, complex128
 
-        reference = [-0.2, float("inf"), float("-inf"), 2.0]
-        a = array([-5.0, 0.0, -0.0, 0.5])
+        inf = float('inf')
+        nan = float('nan')
+        reference = [-0.2, inf, -inf, 2.0, nan]
+        a = array([-5.0, 0.0, -0.0, 0.5, nan])
         b = reciprocal(a)
         for i in range(4):
             assert b[i] == reference[i]
 
+        for dtype in ['int8', 'int16', 'int32', 'int64',
+                      'uint8', 'uint16', 'uint32', 'uint64']:
+            reference = [0, -1, 0, 1, 0]
+            if dtype[0] == 'u':
+                reference[1] = 0
+            # XXX need to fix specialization issue in types.py first
+            #elif dtype == 'int32':
+            #        reference[2] = -2147483648
+            #elif dtype == 'int64':
+            #        reference[2] = -9223372036854775808
+            a = array([-2, -1, 0, 1, 2], dtype)
+            b = reciprocal(a)
+            assert (b == reference).all()
+
     def test_subtract(self):
-        from _numpypy import array, subtract
+        from numpypy import array, subtract
 
         a = array([-5.0, -0.0, 1.0])
         b = array([ 3.0, -2.0,-3.0])
@@ -262,7 +323,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
             assert c[i] == a[i] - b[i]
 
     def test_floorceiltrunc(self):
-        from _numpypy import array, floor, ceil, trunc
+        from numpypy import array, floor, ceil, trunc, complex128
         import math
         ninf, inf = float("-inf"), float("inf")
         a = array([ninf, -1.4, -1.5, -1.0, 0.0, 1.0, 1.4, 0.5, inf])
@@ -274,7 +335,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert all([math.copysign(1, f(-abs(float("nan")))) == -1 for f in floor, ceil, trunc])
 
     def test_copysign(self):
-        from _numpypy import array, copysign
+        from numpypy import array, copysign
 
         reference = [5.0, -0.0, 0.0, -6.0]
         a = array([-5.0, 0.0, 0.0, 6.0])
@@ -290,7 +351,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_exp(self):
         import math
-        from _numpypy import array, exp
+        from numpypy import array, exp
 
         a = array([-5.0, -0.0, 0.0, 12345678.0, float("inf"),
                    -float('inf'), -12343424.0])
@@ -303,11 +364,13 @@ class AppTestUfuncs(BaseNumpyAppTest):
             assert b[i] == res
 
     def test_exp2(self):
-        import math
-        from _numpypy import array, exp2
+        import math 
+        from numpypy import array, exp2
+        inf = float('inf')
+        ninf = -float('inf')
+        nan = float('nan')
 
-        a = array([-5.0, -0.0, 0.0, 2, 12345678.0, float("inf"),
-                   -float('inf'), -12343424.0])
+        a = array([-5.0, -0.0, 0.0, 2, 12345678.0, inf, ninf, -12343424.0])
         b = exp2(a)
         for i in range(len(a)):
             try:
@@ -317,11 +380,14 @@ class AppTestUfuncs(BaseNumpyAppTest):
             assert b[i] == res
 
         assert exp2(3) == 8
-        assert math.isnan(exp2(float("nan")))
+        assert math.isnan(exp2(nan))
 
     def test_expm1(self):
-        import math
-        from _numpypy import array, expm1
+        import math, cmath
+        from numpypy import array, expm1
+        inf = float('inf')
+        ninf = -float('inf')
+        nan = float('nan')
 
         a = array([-5.0, -0.0, 0.0, 12345678.0, float("inf"),
                    -float('inf'), -12343424.0])
@@ -335,9 +401,10 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
         assert expm1(1e-50) == 1e-50
 
+
     def test_sin(self):
         import math
-        from _numpypy import array, sin
+        from numpypy import array, sin
 
         a = array([0, 1, 2, 3, math.pi, math.pi*1.5, math.pi*2])
         b = sin(a)
@@ -345,12 +412,12 @@ class AppTestUfuncs(BaseNumpyAppTest):
             assert b[i] == math.sin(a[i])
 
         a = sin(array([True, False], dtype=bool))
-        assert abs(a[0] - sin(1)) < 1e-7  # a[0] will be less precise
+        assert abs(a[0] - sin(1)) < 1e-3  # a[0] will be very imprecise
         assert a[1] == 0.0
 
     def test_cos(self):
         import math
-        from _numpypy import array, cos
+        from numpypy import array, cos
 
         a = array([0, 1, 2, 3, math.pi, math.pi*1.5, math.pi*2])
         b = cos(a)
@@ -359,7 +426,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_tan(self):
         import math
-        from _numpypy import array, tan
+        from numpypy import array, tan
 
         a = array([0, 1, 2, 3, math.pi, math.pi*1.5, math.pi*2])
         b = tan(a)
@@ -368,7 +435,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_arcsin(self):
         import math
-        from _numpypy import array, arcsin
+        from numpypy import array, arcsin
 
         a = array([-1, -0.5, -0.33, 0, 0.33, 0.5, 1])
         b = arcsin(a)
@@ -382,7 +449,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_arccos(self):
         import math
-        from _numpypy import array, arccos
+        from numpypy import array, arccos
 
         a = array([-1, -0.5, -0.33, 0, 0.33, 0.5, 1])
         b = arccos(a)
@@ -396,7 +463,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_arctan(self):
         import math
-        from _numpypy import array, arctan
+        from numpypy import array, arctan
 
         a = array([-3, -2, -1, 0, 1, 2, 3, float('inf'), float('-inf')])
         b = arctan(a)
@@ -409,7 +476,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_arctan2(self):
         import math
-        from _numpypy import array, arctan2
+        from numpypy import array, arctan2
 
         # From the numpy documentation
         assert (
@@ -424,7 +491,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_sinh(self):
         import math
-        from _numpypy import array, sinh
+        from numpypy import array, sinh
 
         a = array([-1, 0, 1, float('inf'), float('-inf')])
         b = sinh(a)
@@ -433,7 +500,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_cosh(self):
         import math
-        from _numpypy import array, cosh
+        from numpypy import array, cosh
 
         a = array([-1, 0, 1, float('inf'), float('-inf')])
         b = cosh(a)
@@ -442,7 +509,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_tanh(self):
         import math
-        from _numpypy import array, tanh
+        from numpypy import array, tanh
 
         a = array([-1, 0, 1, float('inf'), float('-inf')])
         b = tanh(a)
@@ -451,7 +518,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_arcsinh(self):
         import math
-        from _numpypy import arcsinh
+        from numpypy import arcsinh
 
         for v in [float('inf'), float('-inf'), 1.0, math.e]:
             assert math.asinh(v) == arcsinh(v)
@@ -459,7 +526,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_arccosh(self):
         import math
-        from _numpypy import arccosh
+        from numpypy import arccosh
 
         for v in [1.0, 1.1, 2]:
             assert math.acosh(v) == arccosh(v)
@@ -468,7 +535,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_arctanh(self):
         import math
-        from _numpypy import arctanh
+        from numpypy import arctanh
 
         for v in [.99, .5, 0, -.5, -.99]:
             assert math.atanh(v) == arctanh(v)
@@ -479,7 +546,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_sqrt(self):
         import math
-        from _numpypy import sqrt
+        from numpypy import sqrt
 
         nan, inf = float("nan"), float("inf")
         data = [1, 2, 3, inf]
@@ -490,7 +557,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_square(self):
         import math
-        from _numpypy import square
+        from numpypy import square
 
         nan, inf, ninf = float("nan"), float("inf"), float("-inf")
 
@@ -503,7 +570,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_radians(self):
         import math
-        from _numpypy import radians, array
+        from numpypy import radians, array
         a = array([
             -181, -180, -179,
             181, 180, 179,
@@ -516,7 +583,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_deg2rad(self):
         import math
-        from _numpypy import deg2rad, array
+        from numpypy import deg2rad, array
         a = array([
             -181, -180, -179,
             181, 180, 179,
@@ -529,7 +596,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_degrees(self):
         import math
-        from _numpypy import degrees, array
+        from numpypy import degrees, array
         a = array([
             -181, -180, -179,
             181, 180, 179,
@@ -542,7 +609,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_rad2deg(self):
         import math
-        from _numpypy import rad2deg, array
+        from numpypy import rad2deg, array
         a = array([
             -181, -180, -179,
             181, 180, 179,
@@ -554,13 +621,13 @@ class AppTestUfuncs(BaseNumpyAppTest):
             assert b[i] == math.degrees(a[i])
 
     def test_reduce_errors(self):
-        from _numpypy import sin, add
+        from numpypy import sin, add
 
         raises(ValueError, sin.reduce, [1, 2, 3])
-        raises((ValueError, TypeError), add.reduce, 1)
+        assert add.reduce(1) == 1
 
     def test_reduce_1d(self):
-        from _numpypy import add, maximum, less
+        from numpypy import add, maximum, less
 
         assert less.reduce([5, 4, 3, 2, 1])
         assert add.reduce([1, 2, 3]) == 6
@@ -569,20 +636,21 @@ class AppTestUfuncs(BaseNumpyAppTest):
         raises(ValueError, maximum.reduce, [])
 
     def test_reduceND(self):
-        from _numpypy import add, arange
+        from numpypy import add, arange
         a = arange(12).reshape(3, 4)
         assert (add.reduce(a, 0) == [12, 15, 18, 21]).all()
         assert (add.reduce(a, 1) == [6.0, 22.0, 38.0]).all()
+        raises(ValueError, add.reduce, a, 2)
 
     def test_reduce_keepdims(self):
-        from _numpypy import add, arange
+        from numpypy import add, arange
         a = arange(12).reshape(3, 4)
         b = add.reduce(a, 0, keepdims=True)
         assert b.shape == (1, 4)
         assert (add.reduce(a, 0, keepdims=True) == [12, 15, 18, 21]).all()
 
     def test_bitwise(self):
-        from _numpypy import bitwise_and, bitwise_or, bitwise_xor, arange, array
+        from numpypy import bitwise_and, bitwise_or, bitwise_xor, arange, array
         a = arange(6).reshape(2, 3)
         assert (a & 1 == [[0, 1, 0], [1, 0, 1]]).all()
         assert (a & 1 == bitwise_and(a, 1)).all()
@@ -592,21 +660,23 @@ class AppTestUfuncs(BaseNumpyAppTest):
         raises(TypeError, 'array([1.0]) & 1')
 
     def test_unary_bitops(self):
-        from _numpypy import bitwise_not, invert, array
+        from numpypy import bitwise_not, invert, array
         a = array([1, 2, 3, 4])
         assert (~a == [-2, -3, -4, -5]).all()
         assert (bitwise_not(a) == ~a).all()
         assert (invert(a) == ~a).all()
 
     def test_shift(self):
-        from _numpypy import left_shift, right_shift
+        from numpypy import left_shift, right_shift, bool
 
         assert (left_shift([5, 1], [2, 13]) == [20, 2**13]).all()
         assert (right_shift(10, range(5)) == [10, 5, 2, 1, 0]).all()
+        assert left_shift(bool(1), 3) == left_shift(1, 3)
+        assert right_shift(bool(1), 3) == right_shift(1, 3)
 
     def test_comparisons(self):
         import operator
-        from _numpypy import equal, not_equal, less, less_equal, greater, greater_equal
+        from numpypy import equal, not_equal, less, less_equal, greater, greater_equal
 
         for ufunc, func in [
             (equal, operator.eq),
@@ -630,20 +700,24 @@ class AppTestUfuncs(BaseNumpyAppTest):
             ]:
                 assert ufunc(a, b) == func(a, b)
 
-    def test_count_reduce_items(self):
-        from _numpypy import count_reduce_items, arange
-        a = arange(24).reshape(2, 3, 4)
-        assert count_reduce_items(a) == 24
-        assert count_reduce_items(a, 1) == 3
-        assert count_reduce_items(a, (1, 2)) == 3 * 4
 
-    def test_true_divide(self):
-        from _numpypy import arange, array, true_divide
+    def test_count_nonzero(self):
+        from numpypy import count_nonzero
+        assert count_nonzero(0) == 0
+        assert count_nonzero(1) == 1
+        assert count_nonzero([]) == 0
+        assert count_nonzero([1, 2, 0]) == 2
+        assert count_nonzero([[1, 2, 0], [1, 0, 2]]) == 4
+
+    def test_true_divide_2(self):
+        from numpypy import arange, array, true_divide
         assert (true_divide(arange(3), array([2, 2, 2])) == array([0, 0.5, 1])).all()
 
     def test_isnan_isinf(self):
-        from _numpypy import isnan, isinf, float64, array
+        from numpypy import isnan, isinf, float64, array
         assert isnan(float('nan'))
+        assert not isnan(3)
+        assert not isinf(3)
         assert isnan(float64(float('nan')))
         assert not isnan(3)
         assert isinf(float('inf'))
@@ -656,8 +730,10 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert isinf(array([0.2])).dtype.kind == 'b'
 
     def test_isposinf_isneginf(self):
-        from _numpypy import isneginf, isposinf
+        from numpypy import isneginf, isposinf
         assert isposinf(float('inf'))
+        assert not isposinf(3)
+        assert not isneginf(3)
         assert not isposinf(float('-inf'))
         assert not isposinf(float('nan'))
         assert not isposinf(0)
@@ -669,14 +745,25 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert not isneginf(0.0)
 
     def test_isfinite(self):
-        from _numpypy import isfinite
+        from numpypy import isfinite
+        inf = float('inf')
+        ninf = -float('inf')
+        nan = float('nan')
         assert (isfinite([0, 0.0, 1e50, -1e-50]) ==
             [True, True, True, True]).all()
-        assert (isfinite([float('-inf'), float('inf'), float('-nan'), float('nan')]) ==
+        assert (isfinite([ninf, inf, -nan, nan]) ==
             [False, False, False, False]).all()
+        assert (isfinite([1, 2, 3]) == [True, True, True]).all()
+
+        a = [complex(0, 0), complex(1e50, -1e-50), complex(inf, 0),
+             complex(inf, inf), complex(inf, ninf), complex(0, inf),
+             complex(ninf, ninf), complex(nan, 0), complex(0, nan),
+             complex(nan, nan)]
+        assert (isfinite(a) == [True, True, False, False, False, 
+                        False, False, False, False, False]).all() 
 
     def test_logical_ops(self):
-        from _numpypy import logical_and, logical_or, logical_xor, logical_not
+        from numpypy import logical_and, logical_or, logical_xor, logical_not
 
         assert (logical_and([True, False , True, True], [1, 1, 3, 0])
                 == [True, False, True, False]).all()
@@ -688,7 +775,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_logn(self):
         import math
-        from _numpypy import log, log2, log10
+        from numpypy import log, log2, log10
 
         for log_func, base in [(log, math.e), (log2, 2), (log10, 10)]:
             for v in [float('-nan'), float('-inf'), -1, float('nan')]:
@@ -700,7 +787,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_log1p(self):
         import math
-        from _numpypy import log1p
+        from numpypy import log1p
 
         for v in [float('-nan'), float('-inf'), -2, float('nan')]:
             assert math.isnan(log1p(v))
@@ -711,7 +798,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_power_float(self):
         import math
-        from _numpypy import power, array
+        from numpypy import power, array
         a = array([1., 2., 3.])
         b = power(a, 3)
         for i in range(len(a)):
@@ -739,7 +826,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
 
     def test_power_int(self):
         import math
-        from _numpypy import power, array
+        from numpypy import power, array
         a = array([1, 2, 3])
         b = power(a, 3)
         for i in range(len(a)):
@@ -762,17 +849,31 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert power(12345, -12345) == 0
 
     def test_floordiv(self):
-        from _numpypy import floor_divide, array
+        from numpypy import floor_divide, array
+        import math
         a = array([1., 2., 3., 4., 5., 6., 6.01])
         b = floor_divide(a, 2.5)
         for i in range(len(a)):
             assert b[i] == a[i] // 2.5
 
+        a = array([10+10j, -15-100j, 0+10j], dtype=complex)
+        b = floor_divide(a, 2.5)
+        for i in range(len(a)):
+            assert b[i] == a[i] // 2.5
+        b = floor_divide(a, 2.5+3j)
+        #numpy returns (a.real*b.real + a.imag*b.imag) / abs(b)**2
+        expect = [3., -23., 1.]
+        for i in range(len(a)):
+            assert b[i] == expect[i] 
+        b = floor_divide(a[0], 0.)
+        assert math.isnan(b.real)
+        assert b.imag == 0.
+
     def test_logaddexp(self):
         import math
         import sys
         float_max, float_min = sys.float_info.max, sys.float_info.min
-        from _numpypy import logaddexp
+        from numpypy import logaddexp
 
         # From the numpy documentation
         prob1 = math.log(1e-50)
@@ -798,7 +899,7 @@ class AppTestUfuncs(BaseNumpyAppTest):
         import math
         import sys
         float_max, float_min = sys.float_info.max, sys.float_info.min
-        from _numpypy import logaddexp2
+        from numpypy import logaddexp2
         log2 = math.log(2)
 
         # From the numpy documentation
@@ -820,3 +921,5 @@ class AppTestUfuncs(BaseNumpyAppTest):
         assert logaddexp2(float('-inf'), float('inf')) == float('inf')
         assert logaddexp2(float('inf'), float('-inf')) == float('inf')
         assert logaddexp2(float('inf'), float('inf')) == float('inf')
+
+
