@@ -17,12 +17,12 @@ class Module(MixedModule):
          'expr'         : 'pyparser.expr',
          'issuite'      : 'pyparser.issuite',
          'isexpr'       : 'pyparser.isexpr',
-         'STType'       : 'pyparser.STType',
+         'STType'       : 'pyparser.W_STType',
          'ast2tuple'    : 'pyparser.st2tuple',
          'st2tuple'     : 'pyparser.st2tuple',
          'ast2list'     : 'pyparser.st2list',
          'ast2tuple'    : 'pyparser.st2tuple',
-         'ASTType'      : 'pyparser.STType',
+         'ASTType'      : 'pyparser.W_STType',
          'compilest'    : 'pyparser.compilest',
          'compileast'   : 'pyparser.compilest',
          'ParserError'  : 'space.new_exception_class("parser.ParserError")',
