@@ -1,19 +1,19 @@
-from pypy.interpreter.gateway import unwrap_spec, NoneNotWrapped
-from pypy.rlib import rposix, objectmodel
-from pypy.rlib.objectmodel import specialize
-from pypy.rlib.rarithmetic import r_longlong
-from pypy.rlib.unroll import unrolling_iterable
+from pypy.interpreter.gateway import unwrap_spec
+from rpython.rlib import rposix, objectmodel, rurandom
+from rpython.rlib.objectmodel import specialize
+from rpython.rlib.rarithmetic import r_longlong
+from rpython.rlib.unroll import unrolling_iterable
 from pypy.interpreter.error import OperationError, wrap_oserror, wrap_oserror2
-from pypy.interpreter.error import operationerrfmt
-from pypy.rpython.module.ll_os import RegisterOs
-from pypy.rpython.module import ll_os_stat
-from pypy.rpython.lltypesystem import rffi, lltype
-from pypy.rpython.tool import rffi_platform
-from pypy.translator.tool.cbuild import ExternalCompilationInfo
+from rpython.rtyper.module.ll_os import RegisterOs
+from rpython.rtyper.module import ll_os_stat
 from pypy.module.sys.interp_encoding import getfilesystemencoding
 
-import os, sys
-_WIN = sys.platform == 'win32'
+import os
+import sys
+
+_WIN32 = sys.platform == 'win32'
+if _WIN32:
+    from rpython.rlib.rwin32 import _MAX_ENV
 
 c_int = "c_int"
 
@@ -299,7 +299,7 @@ class StatState(object):
     def __init__(self, space):
         self.stat_float_times = True
 
-def stat_float_times(space, w_value=NoneNotWrapped):
+def stat_float_times(space, w_value=None):
     """stat_float_times([newval]) -> oldval
 
 Determine whether os.[lf]stat represents time stamps as float objects.
@@ -418,7 +418,7 @@ def getcwd(space):
     else:
         return space.wrap(cur)
 
-if sys.platform == 'win32':
+if _WIN32:
     def getcwdu(space):
         """Return the current working directory as a unicode string."""
         try:
@@ -491,17 +491,15 @@ class State:
     def __init__(self, space):
         self.space = space
         self.w_environ = space.newdict()
-        if _WIN:
-            self.cryptProviderPtr = lltype.malloc(
-                rffi.CArray(HCRYPTPROV), 1, zero=True,
-                flavor='raw', immortal=True)
+        self.random_context = rurandom.init_urandom()
     def startup(self, space):
         _convertenviron(space, self.w_environ)
     def _freeze_(self):
         # don't capture the environment in the translated pypy
         self.space.call_method(self.w_environ, 'clear')
-        if _WIN:
-            self.cryptProviderPtr[0] = HCRYPTPROV._default
+        # also reset random_context to a fresh new context (empty so far,
+        # to be filled at run-time by rurandom.urandom())
+        self.random_context = rurandom.init_urandom()
         return True
 
 def get(space):
@@ -515,6 +513,9 @@ def _convertenviron(space, w_env):
 @unwrap_spec(name='str0', value='str0')
 def putenv(space, name, value):
     """Change or add an environment variable."""
+    if _WIN32 and len(name) > _MAX_ENV:
+        raise OperationError(space.w_ValueError, space.wrap(
+                "the environment variable is longer than %d bytes" % _MAX_ENV))
     try:
         os.environ[name] = value
     except OSError, e:
@@ -577,6 +578,16 @@ def chmod(space, w_path, mode):
     except OSError, e:
         raise wrap_oserror2(space, e, w_path)
 
+@unwrap_spec(mode=c_int)
+def fchmod(space, w_fd, mode):
+    """Change the access permissions of the file given by file
+descriptor fd."""
+    fd = space.c_filedescriptor_w(w_fd)
+    try:
+        os.fchmod(fd, mode)
+    except OSError, e:
+        raise wrap_oserror(space, e)
+
 def rename(space, w_old, w_new):
     "Rename a file or directory."
     try:
@@ -840,17 +851,6 @@ second form is used, set the access and modified times to the current time.
             raise
         raise OperationError(space.w_TypeError, space.wrap(msg))
 
-def setsid(space):
-    """setsid() -> pid
-
-    Creates a new session with this process as the leader.
-    """
-    try:
-        result = os.setsid()
-    except OSError, e:
-        raise wrap_oserror(space, e)
-    return space.wrap(result)
-
 def uname(space):
     """ uname() -> (sysname, nodename, release, version, machine)
 
@@ -1080,6 +1080,7 @@ for name in RegisterOs.w_star:
         func = declare_new_w_star(name)
         globals()[name] = func
 
+
 @unwrap_spec(fd=c_int)
 def ttyname(space, fd):
     try:
@@ -1087,9 +1088,10 @@ def ttyname(space, fd):
     except OSError, e:
         raise wrap_oserror(space, e)
 
+
 def confname_w(space, w_name, namespace):
     # XXX slightly non-nice, reuses the sysconf of the underlying os module
-    if space.is_true(space.isinstance(w_name, space.w_basestring)):
+    if space.isinstance_w(w_name, space.w_basestring):
         try:
             num = namespace[space.str_w(w_name)]
         except KeyError:
@@ -1113,28 +1115,41 @@ def fpathconf(space, fd, w_name):
 
 @unwrap_spec(path='str0', uid=c_uid_t, gid=c_gid_t)
 def chown(space, path, uid, gid):
+    """Change the owner and group id of path to the numeric uid and gid."""
     check_uid_range(space, uid)
     check_uid_range(space, gid)
     try:
         os.chown(path, uid, gid)
     except OSError, e:
         raise wrap_oserror(space, e, path)
-    return space.w_None
 
 @unwrap_spec(path='str0', uid=c_uid_t, gid=c_gid_t)
 def lchown(space, path, uid, gid):
+    """Change the owner and group id of path to the numeric uid and gid.
+This function will not follow symbolic links."""
     check_uid_range(space, uid)
     check_uid_range(space, gid)
     try:
         os.lchown(path, uid, gid)
     except OSError, e:
         raise wrap_oserror(space, e, path)
-    return space.w_None
+
+@unwrap_spec(uid=c_uid_t, gid=c_gid_t)
+def fchown(space, w_fd, uid, gid):
+    """Change the owner and group id of the file given by file descriptor
+fd to the numeric uid and gid."""
+    fd = space.c_filedescriptor_w(w_fd)
+    check_uid_range(space, uid)
+    check_uid_range(space, gid)
+    try:
+        os.fchown(fd, uid, gid)
+    except OSError, e:
+        raise wrap_oserror(space, e)
 
 def getloadavg(space):
     try:
         load = os.getloadavg()
-    except OSError, e:
+    except OSError:
         raise OperationError(space.w_OSError,
                              space.wrap("Load averages are unobtainable"))
     return space.newtuple([space.wrap(load[0]),
@@ -1165,72 +1180,14 @@ def nice(space, inc):
         raise wrap_oserror(space, e)
     return space.wrap(res)
 
-if _WIN:
-    from pypy.rlib import rwin32
+@unwrap_spec(n=int)
+def urandom(space, n):
+    """urandom(n) -> str
 
-    eci = ExternalCompilationInfo(
-        includes = ['windows.h', 'wincrypt.h'],
-        libraries = ['advapi32'],
-        )
-
-    class CConfig:
-        _compilation_info_ = eci
-        PROV_RSA_FULL = rffi_platform.ConstantInteger(
-            "PROV_RSA_FULL")
-        CRYPT_VERIFYCONTEXT = rffi_platform.ConstantInteger(
-            "CRYPT_VERIFYCONTEXT")
-
-    globals().update(rffi_platform.configure(CConfig))
-
-    HCRYPTPROV = rwin32.ULONG_PTR
-
-    CryptAcquireContext = rffi.llexternal(
-        'CryptAcquireContextA',
-        [rffi.CArrayPtr(HCRYPTPROV),
-         rwin32.LPCSTR, rwin32.LPCSTR, rwin32.DWORD, rwin32.DWORD],
-        rwin32.BOOL,
-        calling_conv='win',
-        compilation_info=eci)
-
-    CryptGenRandom = rffi.llexternal(
-        'CryptGenRandom',
-        [HCRYPTPROV, rwin32.DWORD, rffi.CArrayPtr(rwin32.BYTE)],
-        rwin32.BOOL,
-        calling_conv='win',
-        compilation_info=eci)
-
-    @unwrap_spec(n=int)
-    def win32_urandom(space, n):
-        """urandom(n) -> str
-
-        Return a string of n random bytes suitable for cryptographic use.
-        """
-
-        if n < 0:
-            raise OperationError(space.w_ValueError,
-                                 space.wrap("negative argument not allowed"))
-
-        provider = get(space).cryptProviderPtr[0]
-        if not provider:
-            # Acquire context.
-            # This handle is never explicitly released. The operating
-            # system will release it when the process terminates.
-            if not CryptAcquireContext(
-                get(space).cryptProviderPtr, None, None,
-                PROV_RSA_FULL, CRYPT_VERIFYCONTEXT):
-                raise rwin32.lastWindowsError("CryptAcquireContext")
-
-            provider = get(space).cryptProviderPtr[0]
-
-        # Get random data
-        buf = lltype.malloc(rffi.CArray(rwin32.BYTE), n,
-                            zero=True, # zero seed
-                            flavor='raw')
-        try:
-            if not CryptGenRandom(provider, n, buf):
-                raise rwin32.lastWindowsError("CryptGenRandom")
-
-            return space.wrap(
-                rffi.charpsize2str(rffi.cast(rffi.CCHARP, buf), n))
-        finally:
-            lltype.free(buf, flavor='raw')
+    Return a string of n random bytes suitable for cryptographic use.
+    """
+    context = get(space).random_context
+    try:
+        return space.wrap(rurandom.urandom(context, n))
+    except OSError, e:
+        raise wrap_oserror(space, e)
