@@ -1,13 +1,12 @@
-from pypy.interpreter.baseobjspace import Wrappable
+from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.typedef import TypeDef, GetSetProperty
-from pypy.interpreter.gateway import NoneNotWrapped
-from pypy.interpreter.gateway import interp2app, unwrap_spec
+from pypy.interpreter.gateway import interp2app, unwrap_spec, WrappedDefault
 from pypy.interpreter.error import OperationError
-from pypy.rlib import rgc
-from pypy.rpython.lltypesystem import rffi, lltype
-from pypy.rpython.tool import rffi_platform
-from pypy.translator.tool.cbuild import ExternalCompilationInfo
-from pypy.translator.platform import platform
+from rpython.rlib import rgc, jit
+from rpython.rtyper.lltypesystem import rffi, lltype
+from rpython.rtyper.tool import rffi_platform
+from rpython.translator.tool.cbuild import ExternalCompilationInfo
+from rpython.translator.platform import platform
 
 import sys
 import weakref
@@ -183,6 +182,8 @@ class Storage:
 
     def clear(self):
         self.next_id = 0
+        self._last_object_id = -1
+        self._last_object = None
         self.storage = {}
 
     @staticmethod
@@ -195,15 +196,23 @@ class Storage:
 
     @staticmethod
     def get_object(id):
-        return global_storage.storage[id]
+        if id == global_storage._last_object_id:
+            return global_storage._last_object
+        result = global_storage.storage[id]
+        global_storage._last_object_id = id
+        global_storage._last_object = result
+        return result
 
     @staticmethod
     def free_nonmoving_id(id):
+        if id == global_storage._last_object_id:
+            global_storage._last_object = None
+            global_storage._last_object_id = -1
         del global_storage.storage[id]
 
 global_storage = Storage()
 
-class CallbackData(Wrappable):
+class CallbackData(W_Root):
     def __init__(self, space, parser):
         self.space = space
         self.parser = weakref.ref(parser)
@@ -279,6 +288,7 @@ for index, (name, params) in enumerate(HANDLERS.items()):
         post_code = ''
 
     src = py.code.Source("""
+    @jit.jit_callback('XML:%(name)s')
     def %(name)s_callback(%(first_arg)s, %(args)s):
         id = rffi.cast(lltype.Signed, %(ll_id)s)
         userdata = global_storage.get_object(id)
@@ -295,14 +305,15 @@ for index, (name, params) in enumerate(HANDLERS.items()):
             w_result = space.call_function(handler, %(wargs)s)
             %(post_code)s
         except OperationError, e:
-            parser._exc_info = e
+            if not parser._exc_info: # don't override an existing exception
+                 parser._exc_info = e
             XML_StopParser(parser.itself, XML_FALSE)
             return %(result_error)s
         return %(result_converter)s
     callback = %(name)s_callback
     """ % locals())
 
-    exec str(src)
+    exec src.compile()
 
     c_name = 'XML_Set' + name
     callback_type = lltype.Ptr(lltype.FuncType(
@@ -325,7 +336,8 @@ def UnknownEncodingHandlerData_callback(ll_userdata, name, info):
     try:
         parser.UnknownEncodingHandler(space, name, info)
     except OperationError, e:
-        parser._exc_info = e
+        if parser._exc_info:
+            parser._exc_info = e
         XML_StopParser(parser.itself, XML_FALSE)
         result = 0
     else:
@@ -404,8 +416,8 @@ class Cache:
     def __init__(self, space):
         self.w_error = space.new_exception_class("pyexpat.ExpatError")
 
-class W_XMLParserType(Wrappable):
 
+class W_XMLParserType(W_Root):
     def __init__(self, space, parser, w_intern):
         self.itself = parser
 
@@ -433,7 +445,10 @@ class W_XMLParserType(Wrappable):
         if XML_ParserFree: # careful with CPython interpreter shutdown
             XML_ParserFree(self.itself)
         if global_storage:
-            global_storage.free_nonmoving_id(self.id)
+            try:
+                global_storage.free_nonmoving_id(self.id)
+            except KeyError:
+                pass    # maybe global_storage.clear() was already called
 
     @unwrap_spec(flag=int)
     def SetParamEntityParsing(self, space, flag):
@@ -445,7 +460,8 @@ XML_PARAM_ENTITY_PARSING_ALWAYS. Returns true if setting the flag
 was successful."""
         XML_SetParamEntityParsing(self.itself, flag)
 
-    def UseForeignDTD(self, space, w_flag=True):
+    @unwrap_spec(w_flag=WrappedDefault(True))
+    def UseForeignDTD(self, space, w_flag):
         """UseForeignDTD([flag])
 Allows the application to provide an artificial external subset if one is
 not specified as part of the document instance.  This readily allows the
@@ -459,8 +475,8 @@ getting the advantage of providing document type information to the parser.
 
     def w_convert(self, space, s):
         if self.returns_unicode:
-            from pypy.interpreter.unicodehelper import PyUnicode_DecodeUTF8
-            return space.wrap(PyUnicode_DecodeUTF8(space, s))
+            from pypy.interpreter.unicodehelper import decode_utf8
+            return space.wrap(decode_utf8(space, s))
         else:
             return space.wrap(s)
 
@@ -625,10 +641,13 @@ Parse XML data.  `isfinal' should be true at end of input."""
     def ParseFile(self, space, w_file):
         """ParseFile(file)
 Parse XML data from file-like object."""
-        # XXX not the more efficient method
-        w_data = space.call_method(w_file, 'read')
-        data = space.str_w(w_data)
-        return self.Parse(space, data, isfinal=True)
+        eof = False
+        while not eof:
+            w_data = space.call_method(w_file, 'read', space.wrap(2048))
+            data = space.str_w(w_data)
+            eof = len(data) == 0
+            w_res = self.Parse(space, data, isfinal=eof)
+        return w_res
 
     @unwrap_spec(base=str)
     def SetBase(self, space, base):
@@ -643,7 +662,7 @@ information passed to the ExternalEntityRefHandler."""
         else:
             context = space.str_w(w_context)
 
-        if space.is_w(w_encoding, space.w_None):
+        if space.is_none(w_encoding):
             encoding = None
         else:
             encoding = space.str_w(w_encoding)
@@ -774,12 +793,12 @@ W_XMLParserType.typedef = TypeDef(
     )
 
 def ParserCreate(space, w_encoding=None, w_namespace_separator=None,
-                 w_intern=NoneNotWrapped):
+                 w_intern=None):
     """ParserCreate([encoding[, namespace_separator]]) -> parser
 Return a new XML parser object."""
-    if space.is_w(w_encoding, space.w_None):
+    if space.is_none(w_encoding):
         encoding = None
-    elif space.is_true(space.isinstance(w_encoding, space.w_str)):
+    elif space.isinstance_w(w_encoding, space.w_str):
         encoding = space.str_w(w_encoding)
     else:
         type_name = space.type(w_encoding).getname(space)
@@ -788,9 +807,9 @@ Return a new XML parser object."""
             space.wrap('ParserCreate() argument 1 must be string or None,'
                        ' not %s' % (type_name,)))
 
-    if space.is_w(w_namespace_separator, space.w_None):
+    if space.is_none(w_namespace_separator):
         namespace_separator = 0
-    elif space.is_true(space.isinstance(w_namespace_separator, space.w_str)):
+    elif space.isinstance_w(w_namespace_separator, space.w_str):
         separator = space.str_w(w_namespace_separator)
         if len(separator) == 0:
             namespace_separator = 0
