@@ -1,5 +1,10 @@
+import sys
 from pypy.module.pypyjit.test_pypy_c.test_00_model import BaseTestPyPyC
 
+if sys.maxint == 2147483647:
+    SHIFT = 31
+else:
+    SHIFT = 63
 
 # XXX review the <Call> descrs to replace some EF=4 with EF=3 (elidable)
 
@@ -22,10 +27,10 @@ class TestString(BaseTestPyPyC):
             i14 = int_lt(i6, i9)
             guard_true(i14, descr=...)
             guard_not_invalidated(descr=...)
-            i16 = int_eq(i6, -9223372036854775808)
+            i16 = int_eq(i6, %d)
             guard_false(i16, descr=...)
             i15 = int_mod(i6, i10)
-            i17 = int_rshift(i15, 63)
+            i17 = int_rshift(i15, %d)
             i18 = int_and(i10, i17)
             i19 = int_add(i15, i18)
             i21 = int_lt(i19, 0)
@@ -44,8 +49,8 @@ class TestString(BaseTestPyPyC):
             guard_true(i32, descr=...)
             i34 = int_add(i6, 1)
             --TICK--
-            jump(p0, p1, p2, p3, p4, p5, i34, p7, p8, i9, i10, p11, i12, p13, descr=...)
-        """)
+            jump(..., descr=...)
+        """ % (-sys.maxint-1, SHIFT))
 
     def test_long(self):
         def main(n):
@@ -62,10 +67,10 @@ class TestString(BaseTestPyPyC):
             i11 = int_lt(i6, i7)
             guard_true(i11, descr=...)
             guard_not_invalidated(descr=...)
-            i13 = int_eq(i6, -9223372036854775808)
+            i13 = int_eq(i6, %d)
             guard_false(i13, descr=...)
             i15 = int_mod(i6, i8)
-            i17 = int_rshift(i15, 63)
+            i17 = int_rshift(i15, %d)
             i18 = int_and(i8, i17)
             i19 = int_add(i15, i18)
             i21 = int_lt(i19, 0)
@@ -95,7 +100,7 @@ class TestString(BaseTestPyPyC):
             guard_false(i43, descr=...)
             i46 = call(ConstClass(ll_startswith__rpy_stringPtr_rpy_stringPtr), p28, ConstPtr(ptr45), descr=<Calli 1 rr EF=0>)
             guard_false(i46, descr=...)
-            p51 = new_with_vtable(21136408)
+            p51 = new_with_vtable(...)
             setfield_gc(p51, _, descr=...)    # 7 setfields, but the order is dict-order-dependent
             setfield_gc(p51, _, descr=...)
             setfield_gc(p51, _, descr=...)
@@ -110,8 +115,8 @@ class TestString(BaseTestPyPyC):
             i58 = int_add_ovf(i6, i57)
             guard_no_overflow(descr=...)
             --TICK--
-            jump(p0, p1, p2, p3, p4, p5, i58, i7, descr=...)
-        """)
+            jump(..., descr=...)
+        """ % (-sys.maxint-1, SHIFT))
 
     def test_str_mod(self):
         def main(n):
@@ -159,7 +164,7 @@ class TestString(BaseTestPyPyC):
             guard_no_overflow(descr=...)
             i40 = int_sub(i4, 1)
             --TICK--
-            jump(p0, p1, p2, p3, i40, i38, descr=...)
+            jump(..., descr=...)
         """)
 
     def test_getattr_promote(self):
@@ -232,3 +237,23 @@ class TestString(BaseTestPyPyC):
         loops = log.loops_by_filename(self.filepath)
         loop, = loops
         loop.match_by_id('callone', '')    # nothing
+
+    def test_lookup_codec(self):
+        log = self.run("""
+        import codecs
+
+        def main(n):
+            for i in xrange(n):
+                codecs.lookup('utf8')
+            return i
+        """, [1000])
+        loop, = log.loops_by_filename(self.filepath)
+        loop.match("""
+        i45 = int_lt(i43, i26)
+        guard_true(i45, descr=...)
+        i46 = int_add(i43, 1)
+        setfield_gc(p15, i46, descr=<FieldS pypy.module.__builtin__.functional.W_XRangeIterator.inst_current 8>)
+        guard_not_invalidated(descr=...)
+        --TICK--
+        jump(..., descr=...)
+        """)
