@@ -1,8 +1,4 @@
 # -*- coding: iso-8859-1 -*-
-import autopath
-from pypy.conftest import option, gettestobjspace
-from py.test import raises
-from pypy.interpreter.gateway import app2interp_temp
 import sys
 
 def test_stdin_exists(space):
@@ -16,7 +12,7 @@ def test_stdout_exists(space):
 class AppTestAppSysTests:
 
     def setup_class(cls):
-        cls.w_appdirect = cls.space.wrap(option.runappdirect)
+        cls.w_appdirect = cls.space.wrap(cls.runappdirect)
         cls.w_filesystemenc = cls.space.wrap(sys.getfilesystemencoding())
 
     def test_sys_in_modules(self):
@@ -130,7 +126,7 @@ class AppTestAppSysTests:
 class AppTestSysModulePortedFromCPython:
 
     def setup_class(cls):
-        cls.w_appdirect = cls.space.wrap(option.runappdirect)
+        cls.w_appdirect = cls.space.wrap(cls.runappdirect)
 
     def test_original_displayhook(self):
         import sys, cStringIO, __builtin__
@@ -359,6 +355,7 @@ class AppTestSysModulePortedFromCPython:
         sys.setrecursionlimit(10000)
         assert sys.getrecursionlimit() == 10000
         sys.setrecursionlimit(oldlimit)
+        raises(OverflowError, sys.setrecursionlimit, 1<<31)
 
     def test_getwindowsversion(self):
         import sys
@@ -541,8 +538,8 @@ class AppTestSysModulePortedFromCPython:
         # be changed.
         assert sys.float_repr_style == "short"
 
-class AppTestCurrentFrames:
 
+class AppTestCurrentFrames:
     def test_current_frames(self):
         try:
             import thread
@@ -558,9 +555,11 @@ class AppTestCurrentFrames:
         assert frames.keys() == [0]
         assert frames[0].f_code.co_name in ('f', '?')
 
+
 class AppTestCurrentFramesWithThread(AppTestCurrentFrames):
-    def setup_class(cls):
-        cls.space = gettestobjspace(usemodules=('thread',))
+    spaceconfig = {
+        "usemodules": ["rctime", "thread"],
+    }
 
     def test_current_frames(self):
         import sys
@@ -600,7 +599,7 @@ class AppTestCurrentFramesWithThread(AppTestCurrentFrames):
 class AppTestSysExcInfoDirect:
 
     def setup_method(self, meth):
-        self.checking = not option.runappdirect
+        self.checking = not self.runappdirect
         if self.checking:
             self.seen = []
             from pypy.module.sys import vm
@@ -614,14 +613,14 @@ class AppTestSysExcInfoDirect:
             vm.exc_info_with_tb = exc_info_with_tb
             vm.exc_info_without_tb = exc_info_without_tb
             #
-            from pypy.rlib import jit
+            from rpython.rlib import jit
             self.old2 = [jit.we_are_jitted]
             jit.we_are_jitted = lambda: True
 
     def teardown_method(self, meth):
         if self.checking:
             from pypy.module.sys import vm
-            from pypy.rlib import jit
+            from rpython.rlib import jit
             vm.exc_info_with_tb = self.old[0]
             vm.exc_info_without_tb = self.old[1]
             jit.we_are_jitted = self.old2[0]
@@ -714,5 +713,4 @@ class AppTestSysExcInfoDirect:
 
 
 class AppTestSysExcInfoDirectCallMethod(AppTestSysExcInfoDirect):
-    def setup_class(cls):
-        cls.space = gettestobjspace(**{"objspace.opcodes.CALL_METHOD": True})
+    spaceconfig = {"objspace.opcodes.CALL_METHOD": True}
