@@ -129,10 +129,10 @@ class AppTestThread(GenericTestThread):
             sys.stderr = StringIO.StringIO()
             thread.start_new_thread(fn3, ())
             self.waitfor(lambda: "ValueError" in sys.stderr.getvalue())
-            result = sys.stderr.getvalue()
-            assert "ValueError" in result
-            assert "hello world" in result
-            assert len(result.splitlines()) == 1
+            result = sys.stderr.getvalue().splitlines()
+            #assert result[0].startswith("Unhandled exception in thread ")
+            assert result[1].startswith("Traceback ")
+            assert result[-1] == "ValueError: hello world"
         finally:
             sys.stderr = prev
 
@@ -220,16 +220,24 @@ class AppTestThread(GenericTestThread):
         import signal
 
         def f():
-            time.sleep(0.5)
-            thread.interrupt_main()
+            for x in range(5):
+                if waiting:
+                    thread.interrupt_main()
+                    return
+                print 'tock...', x  # <-force the GIL to be released, as
+                time.sleep(0.1)    #   time.sleep doesn't do non-translated
 
         def busy_wait():
-            for x in range(1000):
+            waiting.append(None)
+            for x in range(10):
                 print 'tick...', x  # <-force the GIL to be released, as
-                time.sleep(0.01)    #   time.sleep doesn't do non-translated
+                time.sleep(0.1)    #   time.sleep doesn't do non-translated
+            waiting.pop()
 
         # This is normally called by app_main.py
         signal.signal(signal.SIGINT, signal.default_int_handler)
 
-        thread.start_new_thread(f, ())
-        raises(KeyboardInterrupt, busy_wait)
+        for i in range(100):
+            waiting = []
+            thread.start_new_thread(f, ())
+            raises(KeyboardInterrupt, busy_wait)
