@@ -1,20 +1,20 @@
-from pypy.interpreter.baseobjspace import W_Root, ObjSpace
-from pypy.interpreter.baseobjspace import Wrappable, SpaceCache
+from rpython.annotator.model import SomeInstance, s_None
 from pypy.interpreter import argument, gateway
+from pypy.interpreter.baseobjspace import W_Root, ObjSpace, SpaceCache
 from pypy.interpreter.typedef import TypeDef, GetSetProperty
-from pypy.annotation.model import SomeInstance, s_None
-from pypy.rpython.extregistry import ExtRegistryEntry
-from pypy.rpython.lltypesystem import lltype
-from pypy.tool.sourcetools import compile2, func_with_new_name
-from pypy.rlib.unroll import unrolling_iterable
-from pypy.rlib.objectmodel import instantiate, we_are_translated
-from pypy.rlib.nonconst import NonConstant
-from pypy.rlib.rarithmetic import r_uint, r_singlefloat
-from pypy.translator.translator import TranslationContext
+from pypy.objspace.std.stdtypedef import StdTypeDef
+from pypy.objspace.std.sliceobject import W_SliceObject
+from rpython.rlib.objectmodel import instantiate, we_are_translated, specialize
+from rpython.rlib.nonconst import NonConstant
+from rpython.rlib.rarithmetic import r_uint, r_singlefloat
+from rpython.rtyper.extregistry import ExtRegistryEntry
+from rpython.rtyper.lltypesystem import lltype
 from pypy.tool.option import make_config
+from rpython.tool.sourcetools import compile2, func_with_new_name
+from rpython.translator.translator import TranslationContext
 
 
-class W_MyObject(Wrappable):
+class W_MyObject(W_Root):
     typedef = None
 
     def getdict(self, space):
@@ -47,17 +47,21 @@ class W_MyObject(Wrappable):
 
     def int_w(self, space):
         return NonConstant(-42)
-    
+
     def uint_w(self, space):
         return r_uint(NonConstant(42))
-    
+
     def bigint_w(self, space):
-        from pypy.rlib.rbigint import rbigint
+        from rpython.rlib.rbigint import rbigint
         return rbigint.fromint(NonConstant(42))
 
 class W_MyType(W_MyObject):
     def __init__(self):
         self.mro_w = [w_some_obj(), w_some_obj()]
+        self.dict_w = {'__str__': w_some_obj()}
+
+    def get_module(self):
+        return w_some_obj()
 
 def w_some_obj():
     if NonConstant(False):
@@ -99,7 +103,6 @@ class Entry(ExtRegistryEntry):
 
 
 class FakeObjSpace(ObjSpace):
-
     def __init__(self, config=None):
         self._seen_extras = []
         ObjSpace.__init__(self, config=config)
@@ -134,17 +137,27 @@ class FakeObjSpace(ObjSpace):
         is_root(w_start)
         is_root(w_end)
         is_root(w_step)
+        W_SliceObject(w_start, w_end, w_step)
         return w_some_obj()
 
     def newint(self, x):
         return w_some_obj()
 
+    def newlong(self, x):
+        return w_some_obj()
+
     def newfloat(self, x):
         return w_some_obj()
 
     def newcomplex(self, x, y):
         return w_some_obj()
 
+    def newlong_from_rbigint(self, x):
+        return w_some_obj()
+
+    def newseqiter(self, x):
+        return w_some_obj()
+
     def marshal_w(self, w_obj):
         "NOT_RPYTHON"
         raise NotImplementedError
@@ -158,6 +171,8 @@ class FakeObjSpace(ObjSpace):
         if isinstance(x, r_singlefloat):
             self._wrap_not_rpython(x)
         if isinstance(x, list):
+            if x == []: # special case: it is used e.g. in sys/__init__.py
+                return w_some_obj()
             self._wrap_not_rpython(x)
         return w_some_obj()
     wrap._annspecialcase_ = "specialize:argtype(1)"
@@ -224,11 +239,17 @@ class FakeObjSpace(ObjSpace):
 
     def gettypeobject(self, typedef):
         assert typedef is not None
-        return self.fromcache(TypeCache).getorbuild(typedef)
+        see_typedef(self, typedef)
+        return w_some_type()
 
     def type(self, w_obj):
         return w_some_type()
 
+    def isinstance_w(self, w_inst, w_type):
+        is_root(w_inst)
+        is_root(w_type)
+        return NonConstant(True)
+
     def unpackiterable(self, w_iterable, expected_length=-1):
         is_root(w_iterable)
         if expected_length < 0:
@@ -263,7 +284,7 @@ class FakeObjSpace(ObjSpace):
 
     # ----------
 
-    def translates(self, func=None, argtypes=None, **kwds):
+    def translates(self, func=None, argtypes=None, seeobj_w=[], **kwds):
         config = make_config(None, **kwds)
         if func is not None:
             if argtypes is None:
@@ -273,9 +294,12 @@ class FakeObjSpace(ObjSpace):
         t = TranslationContext(config=config)
         self.t = t     # for debugging
         ann = t.buildannotator()
-        ann.policy.allow_someobjects = False
         if func is not None:
             ann.build_types(func, argtypes, complete_now=False)
+        if seeobj_w:
+            def seeme(n):
+                return seeobj_w[n]
+            ann.build_types(seeme, [int], complete_now=False)
         #
         # annotate all _seen_extras, knowing that annotating some may
         # grow the list
@@ -284,8 +308,10 @@ class FakeObjSpace(ObjSpace):
             #print self._seen_extras
             ann.build_types(self._seen_extras[done], [],
                             complete_now=False)
+            ann.complete_pending_blocks()
             done += 1
         ann.complete()
+        assert done == len(self._seen_extras)
         #t.viewcg()
         t.buildrtyper().specialize()
         t.checkgraphs()
@@ -296,7 +322,7 @@ def setup():
                  ObjSpace.ExceptionTable +
                  ['int', 'str', 'float', 'long', 'tuple', 'list',
                   'dict', 'unicode', 'complex', 'slice', 'bool',
-                  'basestring', 'object']):
+                  'basestring', 'object', 'bytearray']):
         setattr(FakeObjSpace, 'w_' + name, w_some_obj())
     FakeObjSpace.w_type = w_some_type()
     #
@@ -325,20 +351,22 @@ setup()
 
 # ____________________________________________________________
 
-class TypeCache(SpaceCache):
-    def build(cache, typedef):
-        assert isinstance(typedef, TypeDef)
-        for value in typedef.rawdict.values():
-            cache.space.wrap(value)
-        return w_some_obj()
+@specialize.memo()
+def see_typedef(space, typedef):
+    assert isinstance(typedef, TypeDef)
+    if not isinstance(typedef, StdTypeDef):
+        for name, value in typedef.rawdict.items():
+            space.wrap(value)
 
 class FakeCompiler(object):
     pass
 FakeObjSpace.default_compiler = FakeCompiler()
 
-class FakeModule(Wrappable):
+
+class FakeModule(W_Root):
     def __init__(self):
         self.w_dict = w_some_obj()
+
     def get(self, name):
         name + "xx"   # check that it's a string
         return w_some_obj()
