@@ -1,62 +1,52 @@
 import operator
-import sys
-from pypy.interpreter import gateway
 from pypy.interpreter.error import OperationError
 from pypy.objspace.std import model, newformat
+from pypy.objspace.std.floattype import float_typedef, W_AbstractFloatObject
 from pypy.objspace.std.multimethod import FailedToImplementArgs
 from pypy.objspace.std.model import registerimplementation, W_Object
 from pypy.objspace.std.register_all import register_all
 from pypy.objspace.std.noneobject import W_NoneObject
 from pypy.objspace.std.longobject import W_LongObject
-from pypy.rlib.rarithmetic import ovfcheck_float_to_int, intmask, LONG_BIT
-from pypy.rlib.rfloat import (
+from rpython.rlib.rarithmetic import ovfcheck_float_to_int, intmask, LONG_BIT
+from rpython.rlib.rfloat import (
     isinf, isnan, isfinite, INFINITY, NAN, copysign, formatd,
-    DTSF_ADD_DOT_0, DTSF_STR_PRECISION)
-from pypy.rlib.rbigint import rbigint
-from pypy.rlib.objectmodel import we_are_translated
-from pypy.rlib import rfloat
-from pypy.tool.sourcetools import func_with_new_name
+    DTSF_ADD_DOT_0, DTSF_STR_PRECISION, float_as_rbigint_ratio)
+from rpython.rlib.rbigint import rbigint
+from rpython.rlib import rfloat
+from rpython.tool.sourcetools import func_with_new_name
 
 
 import math
 from pypy.objspace.std.intobject import W_IntObject
 
-class W_AbstractFloatObject(W_Object):
-    __slots__ = ()
-
-    def is_w(self, space, w_other):
-        from pypy.rlib.longlong2float import float2longlong
-        if not isinstance(w_other, W_AbstractFloatObject):
-            return False
-        if self.user_overridden_class or w_other.user_overridden_class:
-            return self is w_other
-        one = float2longlong(space.float_w(self))
-        two = float2longlong(space.float_w(w_other))
-        return one == two
-
-    def immutable_unique_id(self, space):
-        if self.user_overridden_class:
-            return None
-        from pypy.rlib.longlong2float import float2longlong
-        from pypy.objspace.std.model import IDTAG_FLOAT as tag
-        val = float2longlong(space.float_w(self))
-        b = rbigint.fromrarith_int(val)
-        b = b.lshift(3).or_(rbigint.fromint(tag))
-        return space.newlong_from_rbigint(b)
-
 
 class W_FloatObject(W_AbstractFloatObject):
     """This is a implementation of the app-level 'float' type.
     The constructor takes an RPython float as an argument."""
-    from pypy.objspace.std.floattype import float_typedef as typedef
     _immutable_fields_ = ['floatval']
 
+    typedef = float_typedef
+
     def __init__(w_self, floatval):
         w_self.floatval = floatval
 
     def unwrap(w_self, space):
         return w_self.floatval
 
+    def float_w(self, space):
+        return self.floatval
+
+    def int(self, space):
+        if (type(self) is not W_FloatObject and
+            space.is_overloaded(self, space.w_float, '__int__')):
+            return W_Object.int(self, space)
+        try:
+            value = ovfcheck_float_to_int(self.floatval)
+        except OverflowError:
+            return space.long(self)
+        else:
+            return space.newint(value)
+
     def __repr__(self):
         return "<W_FloatObject(%f)>" % self.floatval
 
@@ -88,24 +78,17 @@ def float__Float(space, w_float1):
     a = w_float1.floatval
     return W_FloatObject(a)
 
-def int__Float(space, w_value):
-    try:
-        value = ovfcheck_float_to_int(w_value.floatval)
-    except OverflowError:
-        return space.long(w_value)
-    else:
-        return space.newint(value)
-
 def long__Float(space, w_floatobj):
     try:
         return W_LongObject.fromfloat(space, w_floatobj.floatval)
     except OverflowError:
-        if isnan(w_floatobj.floatval):
-            raise OperationError(
-                space.w_ValueError,
-                space.wrap("cannot convert float NaN to integer"))
-        raise OperationError(space.w_OverflowError,
-                             space.wrap("cannot convert float infinity to long"))
+        raise OperationError(
+            space.w_OverflowError,
+            space.wrap("cannot convert float infinity to integer"))
+    except ValueError:
+        raise OperationError(space.w_ValueError,
+                             space.wrap("cannot convert float NaN to integer"))
+
 def trunc__Float(space, w_floatobj):
     whole = math.modf(w_floatobj.floatval)[1]
     try:
@@ -115,9 +98,6 @@ def trunc__Float(space, w_floatobj):
     else:
         return space.newint(value)
 
-def float_w__Float(space, w_float):
-    return w_float.floatval
-
 def _char_from_hex(number):
     return "0123456789abcdef"[number]
 
@@ -292,8 +272,6 @@ def hash__Float(space, w_value):
     return space.wrap(_hash_float(space, w_value.floatval))
 
 def _hash_float(space, v):
-    from pypy.objspace.std.longobject import hash__Long
-
     if isnan(v):
         return 0
 
@@ -312,7 +290,7 @@ def _hash_float(space, v):
             # Convert to long and use its hash.
             try:
                 w_lval = W_LongObject.fromfloat(space, v)
-            except OverflowError:
+            except (OverflowError, ValueError):
                 # can't convert to long int -- arbitrary
                 if v < 0:
                     return -271828
@@ -556,27 +534,18 @@ def getnewargs__Float(space, w_float):
 
 def float_as_integer_ratio__Float(space, w_float):
     value = w_float.floatval
-    if isinf(value):
+    try:
+        num, den = float_as_rbigint_ratio(value)
+    except OverflowError:
         w_msg = space.wrap("cannot pass infinity to as_integer_ratio()")
         raise OperationError(space.w_OverflowError, w_msg)
-    elif isnan(value):
+    except ValueError:
         w_msg = space.wrap("cannot pass nan to as_integer_ratio()")
         raise OperationError(space.w_ValueError, w_msg)
-    float_part, exp = math.frexp(value)
-    for i in range(300):
-        if float_part == math.floor(float_part):
-            break
-        float_part *= 2.0
-        exp -= 1
-    w_num = W_LongObject.fromfloat(space, float_part)
-    w_den = space.newlong(1)
-    w_exp = space.newlong(abs(exp))
-    w_exp = space.lshift(w_den, w_exp)
-    if exp > 0:
-        w_num = space.mul(w_num, w_exp)
-    else:
-        w_den = w_exp
-    # Try to return int.
+
+    w_num = space.newlong_from_rbigint(num)
+    w_den = space.newlong_from_rbigint(den)
+    # Try to return int
     return space.newtuple([space.int(w_num), space.int(w_den)])
 
 def float_is_integer__Float(space, w_float):
