@@ -1,17 +1,17 @@
 import __builtin__
 import types
 from pypy.interpreter import special
-from pypy.interpreter.baseobjspace import ObjSpace, Wrappable
+from pypy.interpreter.baseobjspace import ObjSpace, W_Root
 from pypy.interpreter.error import OperationError, operationerrfmt
 from pypy.interpreter.typedef import get_unique_interplevel_subclass
 from pypy.objspace.std import (builtinshortcut, stdtypedef, frame, model,
-                               transparent, callmethod, proxyobject)
+                               transparent, callmethod)
 from pypy.objspace.descroperation import DescrOperation, raiseattrerror
-from pypy.rlib.objectmodel import instantiate, r_dict, specialize, is_annotation_constant
-from pypy.rlib.debug import make_sure_not_resized
-from pypy.rlib.rarithmetic import base_int, widen, maxint, is_valid_int
-from pypy.rlib.objectmodel import we_are_translated
-from pypy.rlib import jit
+from rpython.rlib.objectmodel import instantiate, specialize, is_annotation_constant
+from rpython.rlib.debug import make_sure_not_resized
+from rpython.rlib.rarithmetic import base_int, widen, is_valid_int
+from rpython.rlib.objectmodel import we_are_translated
+from rpython.rlib import jit
 
 # Object imports
 from pypy.objspace.std.boolobject import W_BoolObject
@@ -23,12 +23,11 @@ from pypy.objspace.std.listobject import W_ListObject
 from pypy.objspace.std.longobject import W_LongObject, newlong
 from pypy.objspace.std.noneobject import W_NoneObject
 from pypy.objspace.std.objectobject import W_ObjectObject
-from pypy.objspace.std.ropeobject import W_RopeObject
 from pypy.objspace.std.iterobject import W_SeqIterObject
 from pypy.objspace.std.setobject import W_SetObject, W_FrozensetObject
 from pypy.objspace.std.sliceobject import W_SliceObject
-from pypy.objspace.std.smallintobject import W_SmallIntObject
 from pypy.objspace.std.stringobject import W_StringObject
+from pypy.objspace.std.unicodeobject import W_UnicodeObject
 from pypy.objspace.std.tupleobject import W_AbstractTupleObject
 from pypy.objspace.std.typeobject import W_TypeObject
 
@@ -47,11 +46,9 @@ class StdObjSpace(ObjSpace, DescrOperation):
         self.model = model.StdTypeModel(self.config)
 
         self.FrameClass = frame.build_frame(self)
+        self.StringObjectCls = W_StringObject
 
-        if self.config.objspace.std.withrope:
-            self.StringObjectCls = W_RopeObject
-        else:
-            self.StringObjectCls = W_StringObject
+        self.UnicodeObjectCls = W_UnicodeObject
 
         self._install_multimethods()
 
@@ -129,13 +126,6 @@ class StdObjSpace(ObjSpace, DescrOperation):
         ec._py_repr = None
         return ec
 
-    def createframe(self, code, w_globals, outer_func=None):
-        from pypy.objspace.std.fake import CPythonFakeCode, CPythonFakeFrame
-        if not we_are_translated() and isinstance(code, CPythonFakeCode):
-            return CPythonFakeFrame(self, code, w_globals)
-        else:
-            return ObjSpace.createframe(self, code, w_globals, outer_func)
-
     def gettypefor(self, cls):
         return self.gettypeobject(cls.typedef)
 
@@ -155,8 +145,6 @@ class StdObjSpace(ObjSpace, DescrOperation):
         # annotation (see pypy/annotation/builtin.py)
         if x is None:
             return self.w_None
-        if isinstance(x, model.W_Object):
-            raise TypeError, "attempt to wrap already wrapped object: %s"%(x,)
         if isinstance(x, OperationError):
             raise TypeError, ("attempt to wrap already wrapped exception: %s"%
                               (x,))
@@ -171,15 +159,15 @@ class StdObjSpace(ObjSpace, DescrOperation):
             return wrapunicode(self, x)
         if isinstance(x, float):
             return W_FloatObject(x)
-        if isinstance(x, Wrappable):
+        if isinstance(x, W_Root):
             w_result = x.__spacebind__(self)
             #print 'wrapping', x, '->', w_result
             return w_result
         if isinstance(x, base_int):
             if self.config.objspace.std.withsmalllong:
                 from pypy.objspace.std.smalllongobject import W_SmallLongObject
-                from pypy.rlib.rarithmetic import r_longlong, r_ulonglong
-                from pypy.rlib.rarithmetic import longlongmax
+                from rpython.rlib.rarithmetic import r_longlong, r_ulonglong
+                from rpython.rlib.rarithmetic import longlongmax
                 if (not isinstance(x, r_ulonglong)
                     or x <= r_ulonglong(longlongmax)):
                     return W_SmallLongObject(r_longlong(x))
@@ -238,16 +226,9 @@ class StdObjSpace(ObjSpace, DescrOperation):
             # '__builtin__.Ellipsis' avoids confusion with special.Ellipsis
             return self.w_Ellipsis
 
-        if self.config.objspace.nofaking:
-            raise OperationError(self.w_RuntimeError,
-                                 self.wrap("nofaking enabled: refusing "
-                                           "to wrap cpython value %r" %(x,)))
-        if isinstance(x, type(Exception)) and issubclass(x, Exception):
-            w_result = self.wrap_exception_cls(x)
-            if w_result is not None:
-                return w_result
-        from pypy.objspace.std.fake import fake_object
-        return fake_object(self, x)
+        raise OperationError(self.w_RuntimeError,
+            self.wrap("refusing to wrap cpython value %r" % (x,))
+        )
 
     def wrap_exception_cls(self, x):
         """NOT_RPYTHON"""
@@ -259,7 +240,7 @@ class StdObjSpace(ObjSpace, DescrOperation):
     def wraplong(self, x):
         "NOT_RPYTHON"
         if self.config.objspace.std.withsmalllong:
-            from pypy.rlib.rarithmetic import r_longlong
+            from rpython.rlib.rarithmetic import r_longlong
             try:
                 rx = r_longlong(x)
             except OverflowError:
@@ -272,11 +253,11 @@ class StdObjSpace(ObjSpace, DescrOperation):
 
     def unwrap(self, w_obj):
         """NOT_RPYTHON"""
-        if isinstance(w_obj, Wrappable):
-            return w_obj
         if isinstance(w_obj, model.W_Object):
             return w_obj.unwrap(self)
-        raise model.UnwrapError, "cannot unwrap: %r" % w_obj
+        if isinstance(w_obj, W_Root):
+            return w_obj
+        raise model.UnwrapError("cannot unwrap: %r" % w_obj)
 
     def newint(self, intval):
         return wrapint(self, intval)
@@ -461,6 +442,21 @@ class StdObjSpace(ObjSpace, DescrOperation):
             return w_obj.getitems_str()
         return None
 
+    def listview_unicode(self, w_obj):
+        # note: uses exact type checking for objects with strategies,
+        # and isinstance() for others.  See test_listobject.test_uses_custom...
+        if type(w_obj) is W_ListObject:
+            return w_obj.getitems_unicode()
+        if type(w_obj) is W_DictMultiObject:
+            return w_obj.listview_unicode()
+        if type(w_obj) is W_SetObject or type(w_obj) is W_FrozensetObject:
+            return w_obj.listview_unicode()
+        if isinstance(w_obj, W_UnicodeObject):
+            return w_obj.listview_unicode()
+        if isinstance(w_obj, W_ListObject) and self._uses_list_iter(w_obj):
+            return w_obj.getitems_unicode()
+        return None
+
     def listview_int(self, w_obj):
         if type(w_obj) is W_ListObject:
             return w_obj.getitems_int()
@@ -583,16 +579,8 @@ class StdObjSpace(ObjSpace, DescrOperation):
             self.setitem(w_obj, self.wrap(key), w_value)
 
     def getindex_w(self, w_obj, w_exception, objdescr=None):
-        # Performance shortcut for the common case of w_obj being an int.
-        # If withsmallint is disabled, we check for W_IntObject.
-        # If withsmallint is enabled, we only check for W_SmallIntObject - it's
-        # probably not useful to have a shortcut for W_IntObject at all then.
-        if self.config.objspace.std.withsmallint:
-            if type(w_obj) is W_SmallIntObject:
-                return w_obj.intval
-        else:
-            if type(w_obj) is W_IntObject:
-                return w_obj.intval
+        if type(w_obj) is W_IntObject:
+            return w_obj.intval
         return ObjSpace.getindex_w(self, w_obj, w_exception, objdescr)
 
     def call_method(self, w_obj, methname, *arg_w):
@@ -601,10 +589,6 @@ class StdObjSpace(ObjSpace, DescrOperation):
         else:
             return ObjSpace.call_method(self, w_obj, methname, *arg_w)
 
-    def raise_key_error(self, w_key):
-        e = self.call_function(self.w_KeyError, w_key)
-        raise OperationError(self.w_KeyError, e)
-
     def _type_issubtype(self, w_sub, w_type):
         if isinstance(w_sub, W_TypeObject) and isinstance(w_type, W_TypeObject):
             return self.wrap(w_sub.issubtype(w_type))
@@ -623,10 +607,6 @@ class StdObjSpace(ObjSpace, DescrOperation):
                     return True
         return self.type(w_inst).issubtype(w_type)
 
-    @specialize.arg_or_var(2)
-    def isinstance_w(space, w_inst, w_type):
-        return space._type_isinstance(w_inst, w_type)
-
     def setup_isinstance_cache(self):
         # This assumes that all classes in the stdobjspace implementing a
         # particular app-level type are distinguished by a common base class.
@@ -689,3 +669,8 @@ class StdObjSpace(ObjSpace, DescrOperation):
         if not hasattr(self, "_interplevel_classes"):
             return None # before running initialize
         return self._interplevel_classes.get(w_type, None)
+
+    @specialize.arg(2, 3)
+    def is_overloaded(self, w_obj, tp, method):
+        return (self.lookup(w_obj, method) is not
+                self.lookup_in_type_where(tp, method)[1])
