@@ -1,23 +1,20 @@
-from pypy.interpreter import gateway
+from pypy.interpreter.gateway import interp2app, unwrap_spec, WrappedDefault
 from pypy.objspace.std.stdtypedef import StdTypeDef, SMM
 from pypy.objspace.std.basestringtype import basestring_typedef
 from pypy.objspace.std.register_all import register_all
 
 
 from sys import maxint
-from pypy.rlib.objectmodel import specialize
-from pypy.rlib.jit import we_are_jitted
+from rpython.rlib.objectmodel import specialize
+from rpython.rlib.jit import we_are_jitted
 
 def wrapstr(space, s):
     from pypy.objspace.std.stringobject import W_StringObject
-    from pypy.objspace.std.ropeobject import rope, W_RopeObject
     if space.config.objspace.std.sharesmallstr:
         if space.config.objspace.std.withprebuiltchar:
             # share characters and empty string
             if len(s) <= 1:
                 if len(s) == 0:
-                    if space.config.objspace.std.withrope:
-                        return W_RopeObject.EMPTY
                     return W_StringObject.EMPTY
                 else:
                     s = s[0]     # annotator hint: a single char
@@ -25,44 +22,25 @@ def wrapstr(space, s):
         else:
             # only share the empty string
             if len(s) == 0:
-                if space.config.objspace.std.withrope:
-                    return W_RopeObject.EMPTY
                 return W_StringObject.EMPTY
-    if space.config.objspace.std.withrope:
-        return W_RopeObject(rope.LiteralStringNode(s))
     return W_StringObject(s)
 
 def wrapchar(space, c):
     from pypy.objspace.std.stringobject import W_StringObject
-    from pypy.objspace.std.ropeobject import rope, W_RopeObject
     if space.config.objspace.std.withprebuiltchar and not we_are_jitted():
-        if space.config.objspace.std.withrope:
-            return W_RopeObject.PREBUILT[ord(c)]
         return W_StringObject.PREBUILT[ord(c)]
     else:
-        if space.config.objspace.std.withrope:
-            return W_RopeObject(rope.LiteralStringNode(c))
         return W_StringObject(c)
 
 def sliced(space, s, start, stop, orig_obj):
     assert start >= 0
     assert stop >= 0
-    assert not space.config.objspace.std.withrope
     if start == 0 and stop == len(s) and space.is_w(space.type(orig_obj), space.w_str):
         return orig_obj
-    if space.config.objspace.std.withstrslice:
-        from pypy.objspace.std.strsliceobject import W_StringSliceObject
-        # XXX heuristic, should be improved!
-        if (stop - start) > len(s) * 0.20 + 40:
-            return W_StringSliceObject(s, start, stop)
     return wrapstr(space, s[start:stop])
 
 def joined2(space, str1, str2):
-    assert not space.config.objspace.std.withrope
-    if space.config.objspace.std.withstrjoin:
-        from pypy.objspace.std.strjoinobject import W_StringJoinObject
-        return W_StringJoinObject([str1, str2])
-    elif space.config.objspace.std.withstrbuf:
+    if space.config.objspace.std.withstrbuf:
         from pypy.objspace.std.strbufobject import joined2
         return joined2(str1, str2)
     else:
@@ -290,7 +268,8 @@ register_all(vars(), globals())
 
 # ____________________________________________________________
 
-def descr__new__(space, w_stringtype, w_object=''):
+@unwrap_spec(w_object = WrappedDefault(""))
+def descr__new__(space, w_stringtype, w_object):
     # NB. the default value of w_object is really a *wrapped* empty string:
     #     there is gateway magic at work
     from pypy.objspace.std.stringobject import W_StringObject
@@ -298,20 +277,14 @@ def descr__new__(space, w_stringtype, w_object=''):
     if space.is_w(w_stringtype, space.w_str):
         return w_obj  # XXX might be reworked when space.str() typechecks
     value = space.str_w(w_obj)
-    if space.config.objspace.std.withrope:
-        from pypy.objspace.std.ropeobject import rope, W_RopeObject
-        w_obj = space.allocate_instance(W_RopeObject, w_stringtype)
-        W_RopeObject.__init__(w_obj, rope.LiteralStringNode(value))
-        return w_obj
-    else:
-        w_obj = space.allocate_instance(W_StringObject, w_stringtype)
-        W_StringObject.__init__(w_obj, value)
-        return w_obj
+    w_obj = space.allocate_instance(W_StringObject, w_stringtype)
+    W_StringObject.__init__(w_obj, value)
+    return w_obj
 
 # ____________________________________________________________
 
 str_typedef = StdTypeDef("str", basestring_typedef,
-    __new__ = gateway.interp2app(descr__new__),
+    __new__ = interp2app(descr__new__),
     __doc__ = '''str(object) -> string
 
 Return a nice string representation of the object.
