@@ -1,20 +1,13 @@
-import py
 import sys
-from pypy.interpreter.error import OperationError
-from pypy.objspace.std.dictmultiobject import \
-     W_DictMultiObject, setitem__DictMulti_ANY_ANY, getitem__DictMulti_ANY, \
-     StringDictStrategy, ObjectDictStrategy
+import py
 
-from pypy.conftest import gettestobjspace
-from pypy.conftest import option
+from pypy.objspace.std.dictmultiobject import (W_DictMultiObject,
+    setitem__DictMulti_ANY_ANY, getitem__DictMulti_ANY, StringDictStrategy,
+    ObjectDictStrategy)
 
-class TestW_DictObject:
-
-    def setup_class(cls):
-        cls.space = gettestobjspace()
 
+class TestW_DictObject(object):
     def test_empty(self):
-        space = self.space
         d = self.space.newdict()
         assert not self.space.is_true(d)
         assert type(d.strategy) is not ObjectDictStrategy
@@ -144,33 +137,48 @@ class TestW_DictObject:
 
     def test_listview_str_dict(self):
         w = self.space.wrap
-
         w_d = self.space.newdict()
         w_d.initialize_content([(w("a"), w(1)), (w("b"), w(2))])
-
         assert self.space.listview_str(w_d) == ["a", "b"]
 
+    def test_listview_unicode_dict(self):
+        w = self.space.wrap
+        w_d = self.space.newdict()
+        w_d.initialize_content([(w(u"a"), w(1)), (w(u"b"), w(2))])
+        assert self.space.listview_unicode(w_d) == [u"a", u"b"]
+
     def test_listview_int_dict(self):
         w = self.space.wrap
         w_d = self.space.newdict()
         w_d.initialize_content([(w(1), w("a")), (w(2), w("b"))])
-
         assert self.space.listview_int(w_d) == [1, 2]
 
-    def test_keys_on_string_int_dict(self):
+    def test_keys_on_string_unicode_int_dict(self, monkeypatch):
         w = self.space.wrap
+        
         w_d = self.space.newdict()
         w_d.initialize_content([(w(1), w("a")), (w(2), w("b"))])
-
         w_l = self.space.call_method(w_d, "keys")
         assert sorted(self.space.listview_int(w_l)) == [1,2]
-
+        
+        # make sure that .keys() calls newlist_str for string dicts
+        def not_allowed(*args):
+            assert False, 'should not be called'
+        monkeypatch.setattr(self.space, 'newlist', not_allowed)
+        #
         w_d = self.space.newdict()
         w_d.initialize_content([(w("a"), w(1)), (w("b"), w(6))])
-
         w_l = self.space.call_method(w_d, "keys")
         assert sorted(self.space.listview_str(w_l)) == ["a", "b"]
 
+        # XXX: it would be nice if the test passed without monkeypatch.undo(),
+        # but we need space.newlist_unicode for it
+        monkeypatch.undo() 
+        w_d = self.space.newdict()
+        w_d.initialize_content([(w(u"a"), w(1)), (w(u"b"), w(6))])
+        w_l = self.space.call_method(w_d, "keys")
+        assert sorted(self.space.listview_unicode(w_l)) == [u"a", u"b"]
+
 class AppTest_DictObject:
     def setup_class(cls):
         cls.w_on_pypy = cls.space.wrap("__pypy__" in sys.builtin_module_names)
@@ -454,6 +462,8 @@ class AppTest_DictObject:
         class E(dict):
             pass
         assert isinstance(D.fromkeys([1, 2]), E)
+        assert dict.fromkeys({"a": 2, "b": 3}) == {"a": None, "b": None}
+        assert dict.fromkeys({"a": 2, 1: 3}) == {"a": None, 1: None}
 
     def test_str_uses_repr(self):
         class D(dict):
@@ -775,7 +785,7 @@ class AppTestDictViews:
 
 class AppTestStrategies(object):
     def setup_class(cls):
-        if option.runappdirect:
+        if cls.runappdirect:
             py.test.skip("__repr__ doesn't work on appdirect")
 
     def w_get_strategy(self, obj):
@@ -797,6 +807,16 @@ class AppTestStrategies(object):
         o.a = 1
         assert "StringDictStrategy" in self.get_strategy(d)
 
+    def test_empty_to_unicode(self):
+        d = {}
+        assert "EmptyDictStrategy" in self.get_strategy(d)
+        d[u"a"] = 1
+        assert "UnicodeDictStrategy" in self.get_strategy(d)
+        assert d[u"a"] == 1
+        assert d["a"] == 1
+        assert d.keys() == [u"a"]
+        assert type(d.keys()[0]) is unicode
+
     def test_empty_to_int(self):
         import sys
         d = {}
@@ -832,7 +852,7 @@ class AppTestStrategies(object):
         raises(RuntimeError, list, it)
 
 
-class FakeString(str):
+class FakeWrapper(object):
     hash_count = 0
     def unwrap(self, space):
         self.unwrapped = True
@@ -842,6 +862,12 @@ class FakeString(str):
         self.hash_count += 1
         return str.__hash__(self)
 
+class FakeString(FakeWrapper, str):
+    pass
+
+class FakeUnicode(FakeWrapper, unicode):
+    pass
+
 # the minimal 'space' needed to use a W_DictMultiObject
 class FakeSpace:
     hash_count = 0
@@ -868,6 +894,7 @@ class FakeSpace:
             return str
         return type(w_obj)
     w_str = str
+
     def str_w(self, string):
         assert isinstance(string, str)
         return string
@@ -879,8 +906,9 @@ class FakeSpace:
     def wrap(self, obj):
         return obj
 
-    def isinstance(self, obj, klass):
+    def isinstance_w(self, obj, klass):
         return isinstance(obj, klass)
+    isinstance = isinstance_w
 
     def newtuple(self, l):
         return tuple(l)
@@ -889,6 +917,9 @@ class FakeSpace:
         return W_DictMultiObject.allocate_and_init_instance(
                 self, module=module, instance=instance)
 
+    def view_as_kwargs(self, w_d):
+        return w_d.view_as_kwargs() # assume it's a multidict
+
     def finditem_str(self, w_dict, s):
         return w_dict.getitem_str(s) # assume it's a multidict
 
@@ -911,6 +942,7 @@ class FakeSpace:
     w_bool = bool
     w_float = float
     StringObjectCls = FakeString
+    UnicodeObjectCls = FakeUnicode
     w_dict = W_DictMultiObject
     iter = iter
     fixedview = list
@@ -923,6 +955,7 @@ class Config:
             withcelldict = False
             withmethodcache = False
             withidentitydict = False
+            withmapdict = False
 
 FakeSpace.config = Config()
 
@@ -1035,10 +1068,10 @@ class BaseTestRDictImplementation:
 
     def test_iter(self):
         self.fill_impl()
-        iteratorimplementation = self.impl.iter()
+        iteratorimplementation = self.impl.iteritems()
         items = []
         while 1:
-            item = iteratorimplementation.next()
+            item = iteratorimplementation.next_item()
             if item == (None, None):
                 break
             items.append(item)
@@ -1105,6 +1138,10 @@ class TestStrDictImplementation(BaseTestRDictImplementation):
         assert self.impl.getitem(s) == 1000
         assert s.unwrapped
 
+    def test_view_as_kwargs(self):
+        self.fill_impl()
+        assert self.fakespace.view_as_kwargs(self.impl) == (["fish", "fish2"], [1000, 2000])
+
 ## class TestMeasuringDictImplementation(BaseTestRDictImplementation):
 ##     ImplementionClass = MeasuringDictImplementation
 ##     DevolvedClass = MeasuringDictImplementation
