@@ -1,11 +1,10 @@
 # coding: iso-8859-15
+import py
 import random
 from pypy.objspace.std.listobject import W_ListObject, SizeListStrategy,\
      IntegerListStrategy, ObjectListStrategy
 from pypy.interpreter.error import OperationError
-from pypy.rlib.rarithmetic import is_valid_int
-
-from pypy.conftest import gettestobjspace, option
+from rpython.rlib.rarithmetic import is_valid_int
 
 
 class TestW_ListObject(object):
@@ -202,6 +201,15 @@ class TestW_ListObject(object):
         w_res = self.space.mul(w(n), w_lis)
         assert self.space.eq_w(w_lis3, w_res)
 
+    def test_mul_does_not_clone(self):
+        # only testing right mul at the moment
+        w = self.space.wrap
+        arg = w(2)
+        w_lis = W_ListObject(self.space, [arg])
+        w_lis.clone = None
+        # does not crash
+        self.space.mul(w_lis, w(5))
+
     def test_setitem(self):
         w = self.space.wrap
         w_list = W_ListObject(self.space, [w(5), w(3)])
@@ -402,14 +410,32 @@ class TestW_ListObject(object):
         space.call_method(w_l, 'append', space.w_None)
         assert isinstance(w_l.strategy, ObjectListStrategy)
 
+    def test_newlist_hint(self):
+        space = self.space
+        w_lst = space.newlist_hint(13)
+        assert isinstance(w_lst.strategy, SizeListStrategy)
+        assert w_lst.strategy.sizehint == 13
+
+    def test_find_fast_on_intlist(self, monkeypatch):
+        monkeypatch.setattr(self.space, "eq_w", None)
+        w = self.space.wrap
+        intlist = W_ListObject(self.space, [w(1),w(2),w(3),w(4),w(5),w(6),w(7)])
+        res = intlist.find(w(4), 0, 7)
+        assert res == 3
+        res = intlist.find(w(4), 0, 100)
+        assert res == 3
+        with py.test.raises(ValueError):
+            intlist.find(w(4), 4, 7)
+        with py.test.raises(ValueError):
+            intlist.find(w(4), 0, 2)
 
 class AppTestW_ListObject(object):
     def setup_class(cls):
         import sys
-        on_cpython = (option.runappdirect and
-                            not hasattr(sys, 'pypy_translation_info'))
+        on_cpython = (cls.runappdirect and
+                      not hasattr(sys, 'pypy_translation_info'))
         cls.w_on_cpython = cls.space.wrap(on_cpython)
-        cls.w_runappdirect = cls.space.wrap(option.runappdirect)
+        cls.w_runappdirect = cls.space.wrap(cls.runappdirect)
 
     def test_getstrategyfromlist_w(self):
         l0 = ["a", "2", "a", True]
@@ -922,6 +948,9 @@ class AppTestW_ListObject(object):
     def test_setitem_slice_performance(self):
         # because of a complexity bug, this used to take forever on a
         # translated pypy.  On CPython2.6 -A, it takes around 5 seconds.
+        import platform
+        if platform.machine().startswith('arm'):
+            skip("consumes too much memory for most ARM machines")
         if self.runappdirect:
             count = 16*1024*1024
         else:
@@ -1083,7 +1112,7 @@ class AppTestW_ListObject(object):
                 self.i = i
             def __eq__(self, other):
                 if self.i == 9:
-                    del l[i - 1]
+                    del l[self.i - 1]
                     return True
                 else:
                     return False
@@ -1188,15 +1217,25 @@ class AppTestW_ListObject(object):
         # strategies, to avoid surprizes depending on the strategy.
         class X: pass
         for base, arg in [
-                (list, []), (list, [5]), (list, ['x']), (list, [X]),
-                (set, []),  (set,  [5]), (set,  ['x']), (set, [X]),
+                (list, []), (list, [5]), (list, ['x']), (list, [X]), (list, [u'x']),
+                (set, []),  (set,  [5]), (set,  ['x']), (set, [X]), (set, [u'x']),
                 (dict, []), (dict, [(5,6)]), (dict, [('x',7)]), (dict, [(X,8)]),
+                (dict, [(u'x', 7)]),
                 ]:
             print base, arg
             class SubClass(base):
                 def __iter__(self):
                     return iter("foobar")
-            assert list(SubClass(arg)) == ['f', 'o', 'o', 'b', 'a', 'r']
+            sub = SubClass(arg)
+            assert list(sub) == ['f', 'o', 'o', 'b', 'a', 'r']
+            l = []
+            l.extend(sub)
+            assert l == ['f', 'o', 'o', 'b', 'a', 'r']
+            # test another list strategy
+            l = ['Z']
+            l.extend(sub)
+            assert l == ['Z', 'f', 'o', 'o', 'b', 'a', 'r']
+
             class Sub2(base):
                 pass
             assert list(Sub2(arg)) == list(base(arg))
@@ -1255,11 +1294,20 @@ class AppTestW_ListObject(object):
         assert ([5] >  [N]) is False
         assert ([5] >= [N]) is False
 
-class AppTestForRangeLists(AppTestW_ListObject):
+    def test_resizelist_hint(self):
+        import __pypy__
+        l2 = []
+        __pypy__.resizelist_hint(l2, 100)
+        l1 = [1, 2, 3]
+        l1[:] = l2
+        assert len(l1) == 0
 
-    def setup_class(cls):
-        cls.space = gettestobjspace(**{"objspace.std.withrangelist" :
-                                       True})
+    def test_use_method_for_wrong_object(self):
+        raises(TypeError, list.append.im_func, 1, 2)
+
+
+class AppTestForRangeLists(AppTestW_ListObject):
+    spaceconfig = {"objspace.std.withrangelist": True}
 
     def test_range_simple_backwards(self):
         x = range(5,1)
@@ -1345,11 +1393,38 @@ class AppTestForRangeLists(AppTestW_ListObject):
         l.reverse()
         assert l == [2,1,0]
 
-class AppTestWithoutStrategies(object):
+    def test_issue1266(self):
+        l = range(1)
+        l.pop()
+        # would previously crash
+        l.append(1)
+        assert l == [1]
 
-    def setup_class(cls):
-        cls.space = gettestobjspace(**{"objspace.std.withliststrategies" :
-                                       False})
+        l = range(1)
+        l.pop()
+        # would previously crash
+        l.reverse()
+        assert l == []
+
+    def test_issue1266_ovf(self):
+        import sys
+
+        l = range(0, sys.maxint, sys.maxint)
+        l.append(sys.maxint)
+        # -2 would be next in the range sequence if overflow were
+        # allowed
+        l.append(-2)
+        assert l == [0, sys.maxint, -2]
+        assert -2 in l
+
+        l = range(-sys.maxint, sys.maxint, sys.maxint // 10)
+        item11 = l[11]
+        assert l[::11] == [-sys.maxint, item11]
+        assert item11 in l[::11]
+
+
+class AppTestWithoutStrategies(object):
+    spaceconfig = {"objspace.std.withliststrategies": False}
 
     def test_no_shared_empty_list(self):
         l = []
@@ -1361,10 +1436,7 @@ class AppTestWithoutStrategies(object):
         assert notshared == []
 
 class AppTestListFastSubscr:
-
-    def setup_class(cls):
-        cls.space = gettestobjspace(**{"objspace.std.optimized_list_getitem" :
-                                       True})
+    spaceconfig = {"objspace.std.optimized_list_getitem": True}
 
     def test_getitem(self):
         import operator
