@@ -1,17 +1,9 @@
 import py
-import sys
 from pypy.objspace.std import longobject as lobj
-from pypy.objspace.std.multimethod import FailedToImplement
-from pypy.interpreter.error import OperationError
-from pypy.rlib.rarithmetic import r_uint
-from pypy.rlib.rbigint import rbigint
-from pypy.conftest import gettestobjspace
+from rpython.rlib.rbigint import rbigint
+
 
 class TestW_LongObject:
-
-    def setup_class(cls):
-        cls.space = gettestobjspace(**{"objspace.nofaking": True})
-
     def test_bigint_w(self):
         space = self.space
         fromlong = lobj.W_LongObject.fromlong
@@ -23,9 +15,12 @@ class TestW_LongObject:
         w_obj = space.wrap(123.456)
         space.raises_w(space.w_TypeError, space.bigint_w, w_obj)
 
+        w_obj = fromlong(42)
+        assert space.unwrap(w_obj) == 42
+
     def test_rint_variants(self):
         py.test.skip("XXX broken!")
-        from pypy.rpython.tool.rfficache import platform
+        from rpython.rtyper.tool.rfficache import platform
         space = self.space
         for r in platform.numbertype_to_rclass.values():
             if r is int:
@@ -236,13 +231,13 @@ class AppTestLong:
 
     def test_math_log(self):
         import math
-        raises(ValueError, math.log, 0L) 
-        raises(ValueError, math.log, -1L) 
-        raises(ValueError, math.log, -2L) 
+        raises(ValueError, math.log, 0L)
+        raises(ValueError, math.log, -1L)
+        raises(ValueError, math.log, -2L)
         raises(ValueError, math.log, -(1L << 10000))
-        #raises(ValueError, math.log, 0) 
-        raises(ValueError, math.log, -1) 
-        raises(ValueError, math.log, -2) 
+        #raises(ValueError, math.log, 0)
+        raises(ValueError, math.log, -1)
+        raises(ValueError, math.log, -2)
 
     def test_long(self):
         import sys
@@ -332,3 +327,11 @@ class AppTestLong:
         class A(long): pass
         b = A(5).real
         assert type(b) is long
+
+    def test__int__(self):
+        class A(long):
+            def __int__(self):
+                return 42
+
+        assert int(long(3)) == long(3)
+        assert int(A(13)) == 42
