@@ -3,7 +3,7 @@ The main test for the set implementation is located
 in the stdlibs test/test_set.py which is located in lib-python
 go there and invoke::
 
-    ../../../pypy/bin/py.py test_set.py
+    ../../../pypy/bin/pyinteractive.py test_set.py
 
 This file just contains some basic tests that make sure, the implementation
 is not too wrong.
@@ -15,7 +15,6 @@ from pypy.objspace.std.setobject import newset
 from pypy.objspace.std.setobject import and__Set_Set
 from pypy.objspace.std.setobject import set_intersection__Set
 from pypy.objspace.std.setobject import eq__Set_Set
-from pypy.conftest import gettestobjspace
 from pypy.objspace.std.listobject import W_ListObject
 
 letters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
@@ -85,7 +84,7 @@ class TestW_SetObject:
         assert space.is_true(self.space.eq(result, W_SetObject(space, self.space.wrap(""))))
 
     def test_create_set_from_list(self):
-        from pypy.objspace.std.setobject import ObjectSetStrategy, StringSetStrategy
+        from pypy.objspace.std.setobject import ObjectSetStrategy, StringSetStrategy, UnicodeSetStrategy
         from pypy.objspace.std.floatobject import W_FloatObject
         from pypy.objspace.std.model import W_Object
 
@@ -107,6 +106,12 @@ class TestW_SetObject:
         assert w_set.strategy is self.space.fromcache(StringSetStrategy)
         assert w_set.strategy.unerase(w_set.sstorage) == {"1":None, "2":None, "3":None}
 
+        w_list = self.space.iter(W_ListObject(self.space, [w(u"1"), w(u"2"), w(u"3")]))
+        w_set = W_SetObject(self.space)
+        _initialize_set(self.space, w_set, w_list)
+        assert w_set.strategy is self.space.fromcache(UnicodeSetStrategy)
+        assert w_set.strategy.unerase(w_set.sstorage) == {u"1":None, u"2":None, u"3":None}
+
         w_list = W_ListObject(self.space, [w("1"), w(2), w("3")])
         w_set = W_SetObject(self.space)
         _initialize_set(self.space, w_set, w_list)
@@ -140,7 +145,6 @@ class TestW_SetObject:
 class AppTestAppSetTest:
 
     def setup_class(self):
-        self.space = gettestobjspace()
         w_fakeint = self.space.appexec([], """():
             class FakeInt(object):
                 def __init__(self, value):
@@ -425,6 +429,8 @@ class AppTestAppSetTest:
         s1 = set()
         s1.update(set('abcd'))
         assert s1 == set('abcd')
+        s1 = set([1, 2.0, "3"])
+        s1.update(set(["3", 4, 5.0]))
 
     def test_recursive_repr(self):
         class A(object):
@@ -642,6 +648,21 @@ class AppTestAppSetTest:
         assert set([1,2,3,'5']).difference(set([2,3,4])) == set([1,'5'])
         assert set().difference(set([1,2,3])) == set()
 
+    def test_difference_bug(self):
+        a = set([1,2,3])
+        b = set([])
+        c = a - b
+        c.remove(2)
+        assert c == set([1, 3])
+        assert a == set([1, 2, 3])
+
+        a = set([1,2,3])
+        b = set(["a", "b", "c"])
+        c = a - b
+        c.remove(2)
+        assert c == set([1, 3])
+        assert a == set([1, 2, 3])
+
     def test_intersection_update(self):
         s = set([1,2,3,4,7])
         s.intersection_update([0,1,2,3,4,5,6])
