@@ -1,58 +1,32 @@
+from pypy.interpreter import gateway
+from pypy.interpreter.baseobjspace import W_Root
+from pypy.interpreter.error import OperationError, operationerrfmt
+from pypy.interpreter.function import Function, StaticMethod
+from pypy.interpreter.typedef import weakref_descr, GetSetProperty,\
+     descr_get_dict
+from pypy.interpreter.astcompiler.misc import mangle
 from pypy.objspace.std.model import W_Object
 from pypy.objspace.std.register_all import register_all
-from pypy.interpreter.function import Function, StaticMethod
-from pypy.interpreter import gateway
-from pypy.interpreter.error import OperationError, operationerrfmt
-from pypy.interpreter.typedef import weakref_descr
-from pypy.interpreter.baseobjspace import W_Root
 from pypy.objspace.std.stdtypedef import std_dict_descr, issubtypedef, Member
-from pypy.objspace.std.objecttype import object_typedef
-from pypy.objspace.std import identitydict
-from pypy.rlib.objectmodel import we_are_translated
-from pypy.rlib.objectmodel import current_object_addr_as_int, compute_hash
-from pypy.rlib.jit import promote, elidable_promote, we_are_jitted,\
-     promote_string
-from pypy.rlib.jit import elidable, dont_look_inside, unroll_safe
-from pypy.rlib.rarithmetic import intmask, r_uint
+from pypy.objspace.std.stdtypedef import StdTypeDef
+
+from rpython.rlib.jit import (promote, elidable_promote, we_are_jitted,
+     promote_string, elidable, dont_look_inside, unroll_safe)
+from rpython.rlib.objectmodel import current_object_addr_as_int, compute_hash
+from rpython.rlib.rarithmetic import intmask, r_uint
+
 
 class TypeCell(W_Root):
     def __init__(self, w_value=None):
         self.w_value = w_value
 
+
 def unwrap_cell(space, w_value):
     if (space.config.objspace.std.withtypeversion and
             isinstance(w_value, TypeCell)):
         return w_value.w_value
     return w_value
 
-# from compiler/misc.py
-
-MANGLE_LEN = 256 # magic constant from compile.c
-
-def _mangle(name, klass):
-    if not name.startswith('__'):
-        return name
-    if len(name) + 2 >= MANGLE_LEN:
-        return name
-    if name.endswith('__'):
-        return name
-    try:
-        i = 0
-        while klass[i] == '_':
-            i = i + 1
-    except IndexError:
-        return name
-    klass = klass[i:]
-
-    tlen = len(klass) + len(name)
-    if tlen > MANGLE_LEN:
-        end = len(klass) + MANGLE_LEN-tlen
-        if end < 0:
-            klass = ''     # annotator hint
-        else:
-            klass = klass[:end]
-
-    return "_%s%s" % (klass, name)
 
 class VersionTag(object):
     pass
@@ -84,8 +58,6 @@ COMPARES_BY_IDENTITY = 1
 OVERRIDES_EQ_CMP_OR_HASH = 2
 
 class W_TypeObject(W_Object):
-    from pypy.objspace.std.typetype import type_typedef as typedef
-
     lazyloaders = {} # can be overridden by specific instances
 
     # the version_tag changes if the dict or the inheritance hierarchy changes
@@ -113,10 +85,8 @@ class W_TypeObject(W_Object):
     # for config.objspace.std.withidentitydict
     compares_by_identity_status = UNKNOWN
 
-    # used to cache the type __new__ function if it comes from a builtin type
-    # != 'type', in that case call__Type will also assumes the result
-    # of the __new__ is an instance of the type
-    w_bltin_new = None
+    # used to cache the type's __new__ function
+    w_new_function = None
 
     @dont_look_inside
     def __init__(w_self, space, name, bases_w, dict_w,
@@ -164,7 +134,7 @@ class W_TypeObject(W_Object):
         generic mutation.
         """
         space = w_self.space
-        assert w_self.is_heaptype() or space.config.objspace.std.mutable_builtintypes
+        assert w_self.is_heaptype() or w_self.is_cpytype()
         if (not space.config.objspace.std.withtypeversion and
             not space.config.objspace.std.getattributeshortcut and
             not space.config.objspace.std.withidentitydict and
@@ -181,7 +151,7 @@ class W_TypeObject(W_Object):
                 w_self.compares_by_identity_status = UNKNOWN
 
         if space.config.objspace.std.newshortcut:
-            w_self.w_bltin_new = None
+            w_self.w_new_function = None
 
         if (space.config.objspace.std.withtypeversion
             and w_self._version_tag is not None):
@@ -193,8 +163,7 @@ class W_TypeObject(W_Object):
             w_subclass.mutated(key)
 
     def version_tag(w_self):
-        if (not we_are_jitted() or w_self.is_heaptype() or
-            w_self.space.config.objspace.std.mutable_builtintypes):
+        if not we_are_jitted() or w_self.is_heaptype():
             return w_self._version_tag
         # prebuilt objects cannot get their version_tag changed
         return w_self._pure_version_tag()
@@ -281,7 +250,7 @@ class W_TypeObject(W_Object):
             if attr in w_self.lazyloaders:
                 # very clever next line: it forces the attr string
                 # to be interned.
-                w_attr = space.new_interned_str(attr)
+                space.new_interned_str(attr)
                 loader = w_self.lazyloaders[attr]
                 del w_self.lazyloaders[attr]
                 w_value = loader()
@@ -295,13 +264,13 @@ class W_TypeObject(W_Object):
         return w_self._getdictvalue_no_unwrapping(space, attr)
 
     def setdictvalue(w_self, space, name, w_value):
-        if (not space.config.objspace.std.mutable_builtintypes
-                and not w_self.is_heaptype()):
+        if not w_self.is_heaptype():
             msg = "can't set attributes on type object '%s'"
             raise operationerrfmt(space.w_TypeError, msg, w_self.name)
         if name == "__del__" and name not in w_self.dict_w:
-            msg = "a __del__ method added to an existing type will not be called"
-            space.warn(msg, space.w_RuntimeWarning)
+            msg = ("a __del__ method added to an existing type will not be "
+                   "called")
+            space.warn(space.wrap(msg), space.w_RuntimeWarning)
         if space.config.objspace.std.withtypeversion:
             version_tag = w_self.version_tag()
             if version_tag is not None:
@@ -318,9 +287,8 @@ class W_TypeObject(W_Object):
 
     def deldictvalue(w_self, space, key):
         if w_self.lazyloaders:
-            w_self._freeze_()    # force un-lazification
-        if (not space.config.objspace.std.mutable_builtintypes
-                and not w_self.is_heaptype()):
+            w_self._cleanup_()    # force un-lazification
+        if not w_self.is_heaptype():
             msg = "can't delete attributes on type object '%s'"
             raise operationerrfmt(space.w_TypeError, msg, w_self.name)
         try:
@@ -457,26 +425,23 @@ class W_TypeObject(W_Object):
                 w_self.name, w_subtype.name, w_subtype.name)
         return w_subtype
 
-    def _freeze_(w_self):
+    def _cleanup_(w_self):
         "NOT_RPYTHON.  Forces the lazy attributes to be computed."
         if 'lazyloaders' in w_self.__dict__:
             for attr in w_self.lazyloaders.keys():
                 w_self.getdictvalue(w_self.space, attr)
             del w_self.lazyloaders
-        return False
 
     def getdict(w_self, space): # returning a dict-proxy!
         from pypy.objspace.std.dictproxyobject import DictProxyStrategy
         from pypy.objspace.std.dictmultiobject import W_DictMultiObject
         if w_self.lazyloaders:
-            w_self._freeze_()    # force un-lazification
+            w_self._cleanup_()    # force un-lazification
         strategy = space.fromcache(DictProxyStrategy)
         storage = strategy.erase(w_self)
         return W_DictMultiObject(space, strategy, storage)
 
     def unwrap(w_self, space):
-        if w_self.instancetypedef.fakedcpytype is not None:
-            return w_self.instancetypedef.fakedcpytype
         from pypy.objspace.std.model import UnwrapError
         raise UnwrapError(w_self)
 
@@ -505,7 +470,7 @@ class W_TypeObject(W_Object):
 
     def get_module(w_self):
         space = w_self.space
-        if w_self.is_heaptype() and '__module__' in w_self.dict_w:
+        if w_self.is_heaptype() and w_self.getdictvalue(space, '__module__') is not None:
             return w_self.getdictvalue(space, '__module__')
         else:
             # for non-heap types, CPython checks for a module.name in the
@@ -524,7 +489,7 @@ class W_TypeObject(W_Object):
             mod = '__builtin__'
         else:
             mod = space.str_w(w_mod)
-        if mod !='__builtin__':
+        if mod != '__builtin__':
             return '%s.%s' % (mod, w_self.name)
         else:
             return w_self.name
@@ -568,16 +533,297 @@ class W_TypeObject(W_Object):
                 subclasses_w.append(w_ob)
         return subclasses_w
 
-
     # for now, weakref support for W_TypeObject is hard to get automatically
     _lifeline_ = None
+
     def getweakref(self):
         return self._lifeline_
+
     def setweakref(self, space, weakreflifeline):
         self._lifeline_ = weakreflifeline
+
     def delweakref(self):
         self._lifeline_ = None
 
+def descr__new__(space, w_typetype, w_name, w_bases=None, w_dict=None):
+    "This is used to create user-defined classes only."
+    # XXX check types
+
+    w_typetype = _precheck_for_new(space, w_typetype)
+
+    # special case for type(x)
+    if (space.is_w(space.type(w_typetype), space.w_type) and w_bases is None and
+        w_dict is None):
+        return space.type(w_name)
+    else:
+        return _create_new_type(space, w_typetype, w_name, w_bases, w_dict)
+
+
+def _create_new_type(space, w_typetype, w_name, w_bases, w_dict):
+    # this is in its own function because we want the special case 'type(x)'
+    # above to be seen by the jit.
+    from pypy.objspace.std.typeobject import W_TypeObject
+
+    if w_bases is None or w_dict is None:
+        raise OperationError(space.w_TypeError, space.wrap("type() takes 1 or 3 arguments"))
+
+    bases_w = space.fixedview(w_bases)
+
+    w_winner = w_typetype
+    for base in bases_w:
+        w_typ = space.type(base)
+        if space.is_w(w_typ, space.w_classobj):
+            continue # special-case old-style classes
+        if space.is_true(space.issubtype(w_winner, w_typ)):
+            continue
+        if space.is_true(space.issubtype(w_typ, w_winner)):
+            w_winner = w_typ
+            continue
+        raise OperationError(space.w_TypeError,
+                             space.wrap("metaclass conflict: "
+                                        "the metaclass of a derived class "
+                                        "must be a (non-strict) subclass "
+                                        "of the metaclasses of all its bases"))
+
+    if not space.is_w(w_winner, w_typetype):
+        newfunc = space.getattr(w_winner, space.wrap('__new__'))
+        if not space.is_w(newfunc, space.getattr(space.w_type, space.wrap('__new__'))):
+            return space.call_function(newfunc, w_winner, w_name, w_bases, w_dict)
+        w_typetype = w_winner
+
+    name = space.str_w(w_name)
+    assert isinstance(name, str)
+    dict_w = {}
+    dictkeys_w = space.listview(w_dict)
+    for w_key in dictkeys_w:
+        key = space.str_w(w_key)
+        dict_w[key] = space.getitem(w_dict, w_key)
+    w_type = space.allocate_instance(W_TypeObject, w_typetype)
+    W_TypeObject.__init__(w_type, space, name, bases_w or [space.w_object],
+                          dict_w)
+    w_type.ready()
+    return w_type
+
+def _precheck_for_new(space, w_type):
+    from pypy.objspace.std.typeobject import W_TypeObject
+    if not isinstance(w_type, W_TypeObject):
+        raise operationerrfmt(space.w_TypeError,
+                              "X is not a type object (%s)",
+                              space.type(w_type).getname(space))
+    return w_type
+
+# ____________________________________________________________
+
+def _check(space, w_type, w_msg=None):
+    from pypy.objspace.std.typeobject import W_TypeObject
+    if not isinstance(w_type, W_TypeObject):
+        if w_msg is None:
+            w_msg = space.wrap("descriptor is for 'type'")
+        raise OperationError(space.w_TypeError, w_msg)
+    return w_type
+
+
+def descr_get__name__(space, w_type):
+    w_type = _check(space, w_type)
+    return space.wrap(w_type.name)
+
+def descr_set__name__(space, w_type, w_value):
+    w_type = _check(space, w_type)
+    if not w_type.is_heaptype():
+        raise operationerrfmt(space.w_TypeError,
+                              "can't set %s.__name__", w_type.name)
+    w_type.name = space.str_w(w_value)
+
+def descr_get__mro__(space, w_type):
+    w_type = _check(space, w_type)
+    return space.newtuple(w_type.mro_w)
+
+def descr_mro(space, w_type):
+    """Return a type's method resolution order."""
+    w_type = _check(space, w_type, space.wrap("expected type"))
+    return space.newlist(w_type.compute_default_mro())
+
+def descr_get__bases__(space, w_type):
+    w_type = _check(space, w_type)
+    return space.newtuple(w_type.bases_w)
+
+def mro_subclasses(space, w_type, temp):
+    from pypy.objspace.std.typeobject import W_TypeObject, compute_mro
+    temp.append((w_type, w_type.mro_w))
+    compute_mro(w_type)
+    for w_sc in w_type.get_subclasses():
+        assert isinstance(w_sc, W_TypeObject)
+        mro_subclasses(space, w_sc, temp)
+
+def descr_set__bases__(space, w_type, w_value):
+    # this assumes all app-level type objects are W_TypeObject
+    from pypy.objspace.std.typeobject import (W_TypeObject, get_parent_layout,
+        check_and_find_best_base, is_mro_purely_of_types)
+
+    w_type = _check(space, w_type)
+    if not w_type.is_heaptype():
+        raise operationerrfmt(space.w_TypeError,
+                              "can't set %s.__bases__", w_type.name)
+    if not space.isinstance_w(w_value, space.w_tuple):
+        raise operationerrfmt(space.w_TypeError,
+                              "can only assign tuple to %s.__bases__, not %s",
+                              w_type.name,
+                              space.type(w_value).getname(space))
+    newbases_w = space.fixedview(w_value)
+    if len(newbases_w) == 0:
+        raise operationerrfmt(space.w_TypeError,
+                    "can only assign non-empty tuple to %s.__bases__, not ()",
+                              w_type.name)
+
+    for w_newbase in newbases_w:
+        if isinstance(w_newbase, W_TypeObject):
+            if w_type in w_newbase.compute_default_mro():
+                raise OperationError(space.w_TypeError,
+                                     space.wrap("a __bases__ item causes"
+                                                " an inheritance cycle"))
+
+    w_oldbestbase = check_and_find_best_base(space, w_type.bases_w)
+    w_newbestbase = check_and_find_best_base(space, newbases_w)
+    oldlayout = w_oldbestbase.get_full_instance_layout()
+    newlayout = w_newbestbase.get_full_instance_layout()
+
+    if oldlayout != newlayout:
+        raise operationerrfmt(space.w_TypeError,
+                           "__bases__ assignment: '%s' object layout"
+                           " differs from '%s'",
+                           w_newbestbase.getname(space),
+                           w_oldbestbase.getname(space))
+
+    # invalidate the version_tag of all the current subclasses
+    w_type.mutated(None)
+
+    # now we can go ahead and change 'w_type.bases_w'
+    saved_bases_w = w_type.bases_w
+    temp = []
+    try:
+        for w_oldbase in saved_bases_w:
+            if isinstance(w_oldbase, W_TypeObject):
+                w_oldbase.remove_subclass(w_type)
+        w_type.bases_w = newbases_w
+        for w_newbase in newbases_w:
+            if isinstance(w_newbase, W_TypeObject):
+                w_newbase.add_subclass(w_type)
+        # try to recompute all MROs
+        mro_subclasses(space, w_type, temp)
+    except:
+        for cls, old_mro in temp:
+            cls.mro_w = old_mro
+        w_type.bases_w = saved_bases_w
+        raise
+    if (space.config.objspace.std.withtypeversion and
+        w_type.version_tag() is not None and
+        not is_mro_purely_of_types(w_type.mro_w)):
+        # Disable method cache if the hierarchy isn't pure.
+        w_type._version_tag = None
+        for w_subclass in w_type.get_subclasses():
+            if isinstance(w_subclass, W_TypeObject):
+                w_subclass._version_tag = None
+    assert w_type.w_same_layout_as is get_parent_layout(w_type)  # invariant
+
+def descr__base(space, w_type):
+    from pypy.objspace.std.typeobject import find_best_base
+    w_type = _check(space, w_type)
+    return find_best_base(space, w_type.bases_w)
+
+def descr__doc(space, w_type):
+    if space.is_w(w_type, space.w_type):
+        return space.wrap("""type(object) -> the object's type
+type(name, bases, dict) -> a new type""")
+    w_type = _check(space, w_type)
+    if not w_type.is_heaptype():
+        return w_type.w_doc
+    w_result = w_type.getdictvalue(space, '__doc__')
+    if w_result is None:
+        return space.w_None
+    else:
+        return space.get(w_result, space.w_None, w_type)
+
+def descr__flags(space, w_type):
+    from copy_reg import _HEAPTYPE
+    _CPYTYPE = 1 # used for non-heap types defined in C
+    _ABSTRACT = 1 << 20
+    #
+    w_type = _check(space, w_type)
+    flags = 0
+    if w_type.flag_heaptype:
+        flags |= _HEAPTYPE
+    if w_type.flag_cpytype:
+        flags |= _CPYTYPE
+    if w_type.flag_abstract:
+        flags |= _ABSTRACT
+    return space.wrap(flags)
+
+def descr_get__module(space, w_type):
+    w_type = _check(space, w_type)
+    return w_type.get_module()
+
+def descr_set__module(space, w_type, w_value):
+    w_type = _check(space, w_type)
+    w_type.setdictvalue(space, '__module__', w_value)
+
+def descr_get___abstractmethods__(space, w_type):
+    w_type = _check(space, w_type)
+    # type itself has an __abstractmethods__ descriptor (this). Don't return it
+    if not space.is_w(w_type, space.w_type):
+        w_result = w_type.getdictvalue(space, "__abstractmethods__")
+        if w_result is not None:
+            return w_result
+    raise OperationError(space.w_AttributeError,
+                         space.wrap("__abstractmethods__"))
+
+def descr_set___abstractmethods__(space, w_type, w_new):
+    w_type = _check(space, w_type)
+    w_type.setdictvalue(space, "__abstractmethods__", w_new)
+    w_type.set_abstract(space.is_true(w_new))
+
+def descr_del___abstractmethods__(space, w_type):
+    w_type = _check(space, w_type)
+    if not w_type.deldictvalue(space, "__abstractmethods__"):
+        raise OperationError(space.w_AttributeError,
+                             space.wrap("__abstractmethods__"))
+    w_type.set_abstract(False)
+
+def descr___subclasses__(space, w_type):
+    """Return the list of immediate subclasses."""
+    w_type = _check(space, w_type)
+    return space.newlist(w_type.get_subclasses())
+
+# ____________________________________________________________
+
+@gateway.unwrap_spec(w_obj=W_TypeObject, w_sub=W_TypeObject)
+def type_issubtype(w_obj, space, w_sub):
+    return space.newbool(w_sub.issubtype(w_obj))
+
+@gateway.unwrap_spec(w_obj=W_TypeObject)
+def type_isinstance(w_obj, space, w_inst):
+    return space.newbool(space.type(w_inst).issubtype(w_obj))
+
+type_typedef = StdTypeDef("type",
+    __new__ = gateway.interp2app(descr__new__),
+    __name__ = GetSetProperty(descr_get__name__, descr_set__name__),
+    __bases__ = GetSetProperty(descr_get__bases__, descr_set__bases__),
+    __base__ = GetSetProperty(descr__base),
+    __mro__ = GetSetProperty(descr_get__mro__),
+    __dict__ = GetSetProperty(descr_get_dict),
+    __doc__ = GetSetProperty(descr__doc),
+    mro = gateway.interp2app(descr_mro),
+    __flags__ = GetSetProperty(descr__flags),
+    __module__ = GetSetProperty(descr_get__module, descr_set__module),
+    __abstractmethods__ = GetSetProperty(descr_get___abstractmethods__,
+                                         descr_set___abstractmethods__,
+                                         descr_del___abstractmethods__),
+    __subclasses__ = gateway.interp2app(descr___subclasses__),
+    __weakref__ = weakref_descr,
+    __instancecheck__ = gateway.interp2app(type_isinstance),
+    __subclasscheck__ = gateway.interp2app(type_issubtype),
+    )
+W_TypeObject.typedef = type_typedef
+
 # ____________________________________________________________
 # Initialization of type objects
 
@@ -701,9 +947,12 @@ def create_all_slots(w_self, hasoldstylebase):
             else:
                 create_slot(w_self, slot_name)
     wantdict = wantdict or hasoldstylebase
-    if wantdict: create_dict_slot(w_self)
-    if wantweakref: create_weakref_slot(w_self)
-    if '__del__' in dict_w: w_self.needsdel = True
+    if wantdict:
+        create_dict_slot(w_self)
+    if wantweakref:
+        create_weakref_slot(w_self)
+    if '__del__' in dict_w:
+        w_self.needsdel = True
 
 def create_slot(w_self, slot_name):
     space = w_self.space
@@ -711,7 +960,7 @@ def create_slot(w_self, slot_name):
         raise OperationError(space.w_TypeError,
                              space.wrap('__slots__ must be identifiers'))
     # create member
-    slot_name = _mangle(slot_name, w_self.name)
+    slot_name = mangle(slot_name, w_self.name)
     if slot_name not in w_self.dict_w:
         # Force interning of slot names.
         slot_name = space.str_w(space.new_interned_str(slot_name))
@@ -825,28 +1074,23 @@ def call__Type(space, w_type, __args__):
     promote(w_type)
     # invoke the __new__ of the type
     if not we_are_jitted():
-        # note that the annotator will figure out that w_type.w_bltin_new can
-        # only be None if the newshortcut config option is not set
-        w_bltin_new = w_type.w_bltin_new
+        # note that the annotator will figure out that w_type.w_new_function
+        # can only be None if the newshortcut config option is not set
+        w_newfunc = w_type.w_new_function
     else:
         # for the JIT it is better to take the slow path because normal lookup
-        # is nicely optimized, but the w_type.w_bltin_new attribute is not
+        # is nicely optimized, but the w_type.w_new_function attribute is not
         # known to the JIT
-        w_bltin_new = None
-    call_init = True
-    if w_bltin_new is not None:
-        w_newobject = space.call_obj_args(w_bltin_new, w_type, __args__)
-    else:
+        w_newfunc = None
+    if w_newfunc is None:
         w_newtype, w_newdescr = w_type.lookup_where('__new__')
         w_newfunc = space.get(w_newdescr, w_type)
         if (space.config.objspace.std.newshortcut and
             not we_are_jitted() and
-            isinstance(w_newtype, W_TypeObject) and
-            not w_newtype.is_heaptype() and
-            not space.is_w(w_newtype, space.w_type)):
-            w_type.w_bltin_new = w_newfunc
-        w_newobject = space.call_obj_args(w_newfunc, w_type, __args__)
-        call_init = space.isinstance_w(w_newobject, w_type)
+            isinstance(w_newtype, W_TypeObject)):
+            w_type.w_new_function = w_newfunc
+    w_newobject = space.call_obj_args(w_newfunc, w_type, __args__)
+    call_init = space.isinstance_w(w_newobject, w_type)
 
     # maybe invoke the __init__ of the type
     if (call_init and not (space.is_w(w_type, space.w_type) and
@@ -865,24 +1109,17 @@ def _issubtype(w_sub, w_type):
 def _pure_issubtype(w_sub, w_type, version_tag1, version_tag2):
     return _issubtype(w_sub, w_type)
 
-def issubtype__Type_Type(space, w_type, w_sub):
-    return space.newbool(w_sub.issubtype(w_type))
-
-def isinstance__Type_ANY(space, w_type, w_inst):
-    return space.newbool(space.type(w_inst).issubtype(w_type))
-
 def repr__Type(space, w_obj):
     w_mod = w_obj.get_module()
     if not space.isinstance_w(w_mod, space.w_str):
         mod = None
     else:
         mod = space.str_w(w_mod)
-    if (not w_obj.is_heaptype() or
-        (mod == '__builtin__' or mod == 'exceptions')):
+    if not w_obj.is_heaptype():
         kind = 'type'
     else:
         kind = 'class'
-    if mod is not None and mod !='__builtin__':
+    if mod is not None and mod != '__builtin__':
         return space.wrap("<%s '%s.%s'>" % (kind, mod, w_obj.name))
     else:
         return space.wrap("<%s '%s'>" % (kind, w_obj.name))
@@ -901,7 +1138,7 @@ def getattr__Type_ANY(space, w_type, w_name):
         # __get__(None, type): turns e.g. functions into unbound methods
         return space.get(w_value, space.w_None, w_type)
     if w_descr is not None:
-        return space.get(w_descr,w_type)
+        return space.get(w_descr, w_type)
     raise operationerrfmt(space.w_AttributeError,
                           "type object '%s' has no attribute '%s'",
                           w_type.name, name)
@@ -909,6 +1146,7 @@ def getattr__Type_ANY(space, w_type, w_name):
 def eq__Type_Type(space, w_self, w_other):
     return space.is_(w_self, w_other)
 
+
 # ____________________________________________________________
 
 
@@ -947,7 +1185,7 @@ def compute_C3_mro(space, cls):
             return mro_error(space, orderlists)  # no candidate found
         assert candidate not in order
         order.append(candidate)
-        for i in range(len(orderlists)-1, -1, -1):
+        for i in range(len(orderlists) - 1, -1, -1):
             if orderlists[i][0] is candidate:
                 del orderlists[i][0]
                 if len(orderlists[i]) == 0:
