@@ -31,12 +31,6 @@ class Test_DescrOperation:
 
 
 class AppTest_Descroperation:
-    OPTIONS = {}
-
-    def setup_class(cls):
-        from pypy import conftest
-        cls.space = conftest.gettestobjspace(**cls.OPTIONS)
-
     def test_special_methods(self):
         class OldStyle:
             pass
@@ -658,7 +652,7 @@ class AppTest_Descroperation:
         class X(object):
             def __len__(self): return 1L
             __nonzero__ = __len__
-        assert X()
+        raises(TypeError, bool, X())  # must return bool or int, not long
         del X.__nonzero__
         assert X()
 
@@ -668,6 +662,7 @@ class AppTest_Descroperation:
             def __len__(self):
                 return sys.maxsize + 1
         raises(OverflowError, len, X())
+        raises(OverflowError, bool, X())
 
     def test_len_underflow(self):
         import sys
@@ -675,10 +670,12 @@ class AppTest_Descroperation:
             def __len__(self):
                 return -1
         raises(ValueError, len, X())
+        raises(ValueError, bool, X())
         class Y(object):
             def __len__(self):
                 return -1L
         raises(ValueError, len, Y())
+        raises(ValueError, bool, Y())
 
     def test_len_custom__int__(self):
         class X(object):
@@ -691,8 +688,12 @@ class AppTest_Descroperation:
 
         l = len(X(3.0))
         assert l == 3 and type(l) is int
+        assert X(3.0)
+        assert not X(0.0)
         l = len(X(X(2)))
         assert l == 2 and type(l) is int
+        assert X(X(2))
+        assert not X(X(0))
 
     def test_bool___contains__(self):
         class X(object):
@@ -721,4 +722,4 @@ class AppTest_Descroperation:
             
 
 class AppTestWithBuiltinShortcut(AppTest_Descroperation):
-    OPTIONS = {'objspace.std.builtinshortcut': True}
+    spaceconfig = {'objspace.std.builtinshortcut': True}
