@@ -1,4 +1,3 @@
-import autopath
 import py
 from pypy.interpreter import gateway, pycode
 from pypy.interpreter.error import OperationError
@@ -207,7 +206,7 @@ def _exc_info(space, err):
             self.type, self.value, self.traceback = sys.exc_info()
 
     return _ExceptionInfo
-""")    
+""")
     try:
         return space.call_function(space._w_ExceptionInfo)
     finally:
@@ -216,7 +215,7 @@ def _exc_info(space, err):
 def pypyraises(space, w_ExpectedException, w_expr, __args__):
     """A built-in function providing the equivalent of py.test.raises()."""
     args_w, kwds_w = __args__.unpack()
-    if space.is_true(space.isinstance(w_expr, space.w_str)):
+    if space.isinstance_w(w_expr, space.w_str):
         if args_w:
             raise OperationError(space.w_TypeError,
                                  space.wrap("raises() takes no argument "
